#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 20:20:44 2020.

@author: duverne
"""

import numpy as np
import numpy.polynomial.polynomial as pl
from astropy.coordinates import SkyCoord
from astropy.stats import sigma_clip
from astropy.table import Table, join, Column
from astropy import units as u
from astroquery.vizier import Vizier
from astroquery.xmatch import XMatch
from muphoten.utils import getpath
from muphoten.photometry import get_source

path_muphoten = getpath()
# from muphoten.utils import CataList
### Starting with some useful functions to manipulate the band and catalog
def u_sdss():
    return ['u', "u'", 'sdss u', 'SDSS u', "sloanu"]

def g_sdss():
    return ['g', "g'", 'sdss g', 'SDSS g', "G'", 'G', "sloang", 'green',
            'sloanG', 'TG', "sG"]

def r_sdss():
    return ['r', "r'", 'sdss r', 'SDSS r', "R'", "sloanr", "sR", "sloanR",
            "SDSSrp+", 'TR', 'SR']

def i_sdss():
    return ['i', "i'", 'sdss i', 'SDSS i', "I'", "sloani","SDSS-i'"]

def z_sdss():
    return ['z', "z'", 'sdss z', 'SDSS z', "Z'", "Z", "sloanz"]

def U_jc():
    return ['U', 'UMag']

def B_jc():
    return ['B', 'BMag']

def V_jc():
    return ['V', 'VMag', '                V']

def R_jc():
    return ['R', 'RMag', 'Rc', '                R', 'Free_R_Free']

def I_jc():
    return ['I', 'IMag', 'Ic']

def c():
    return  ['C', 'c', 'Clear', 'clear', 'cmag',
             'L', 'l', 'Luminance', 'luminance', 'LUM', 'Lum']

def catalog_choice(header):
    """
    Determines the optimal catalog for the calibration.

    Parameters
    ----------
    header : Fits file header.
        Fits file header that have to containn the astrometric informations
        and the band.

    Raises
    ------
    KeyError
        If astrometric keyword or band information
        are not given in the header.
        Either perform astrometric calibation or/and
        use the Muphoten comand line to set the band.

    Returns
    -------
    catalog : str
        catalog index used for the image calibration.

    """

    # Getting astrometric informations.
    if ('CRVAL1' or 'CRVAL2') not in header:
        print
        raise KeyError('No astrometric information found.'
                       'Proceed to astrometric calibration.')
    else:
        ra = header['CRVAL1']
        dec = header['CRVAL2']
    # Getting the band information.
    if 'FILTER' not in header:
        raise KeyError('No filter found in header.')
    else:
        band = header['FILTER']

    # Choosing the catalog.
    rad_deg = 1. * u.arcmin # Check if data are available at the position
    field = SkyCoord(ra, dec, unit=(u.deg, u.deg), frame='icrs')
    z_band = z_sdss()
    u_band = u_sdss()
    B_band = B_jc()
    U_band = U_jc()
    all_u_band = u_sdss() + U_jc()

    if (float(dec) > -30. and band not in all_u_band):
        catalog = 'II/349/ps1' #Use of Pan-STARRS DR 1 as much as possible
        system = 'ps'
        cata_name = 'Pan-STARRS'
    elif Vizier.query_region(field,
                             width=rad_deg,
                             height=rad_deg,
                             catalog="V/147/sdss12")[0]:
        catalog = "V/147/sdss12" # Try SDSS if pan-STARRS is not available
        system = 'sdss'
        cata_name = 'SDSS'

    elif band not in u_band + z_band + U_band + B_band:
        catalog = "I/345/gaia2" # It is impossible to convert GAIA's bands into
                                # SDSS z, or u, nor in U & B jonhson-cousins.
                                # Otherwise, as GAIA is an all sky
        system = 'gaia'         # catalog, it is a good option to calibrate.
        cata_name = 'Gaia'
    elif band : # Else for B, it is possible to use USNO
        catalog = "I/284/out"
        system = 'usno'
        cata_name = 'USNO-B1'
    return catalog, system, cata_name

def CataList():
    cataList = ['Pan-STARRS', 'USNO-B1', 'Gaia', 'SDSS', 'skm']
    return cataList

def getCata(cata_name):
    """
    Return the catalog index used by Vizier and
    the photometric system of the survey.

    Parameters
    ----------
    cata_name : str
        Name of the used catalog.

    Raises
    ------
    KeyError
        If the given catalog name is not in the available one in Muphoten.

    Returns
    -------
    catalog : str
        The catalog index for Vizier request.

    """
    catalist = CataList()
    if cata_name not in catalist:
        raise KeyError('{} not in the avalaible catalogs.'
                       'The available ones are : {}'.format(cata_name,
                                                            catalist))
    else:
        if cata_name == 'Pan-STARRS':
            catalog = 'II/349/ps1'
            system = 'ps'
        elif cata_name == 'USNO-B1':
            catalog = 'I/284/out'
            system = 'usno'
        elif cata_name == 'Gaia':
            catalog = 'I/345/gaia2'
            system = 'gaia'
        elif cata_name == 'skm':
            catalog = 'II/358/smss'
            system = 'skm'
        else:
            catalog = "V/147/sdss12"
            system ='sdss'

    return catalog, system

def get_photmetric_system(band):
    """
    Get the photometric system of the instrument that acquired the image.

    Parameters
    ----------
    band : str
        Name of the filter used top acquire the image.

    Returns
    -------
    system : str
        Name of the sytem in the form use by muphoten.
        jc : Johnson-Cousins (Uc, Bc, vc, Rc, Ic)
        sdss : Sloan Digital Sky Survey
               (u, g, r, i, z - only used for th u band)
        ps : Pan-STARRS (g,r , i, z, y)
        other : Any other filter, most probably unfiltered (ie : clear) or
                Luminance, very close to clear image, with cuts in soft UV
                and NIR
    """
    if band in ['UMag', 'BMag', 'VMag', 'RMag','IMag']:
        system = ['jc']
    elif band in ['umag']:
        system = ['sdss', 'skm']
    elif band in ['gmag', 'rmag', 'imag', 'zmag']:
        system = ['ps', 'sdss', 'skm']
    else:
        system = ['other']

    return system

def get_filter(header, band=None):
    """
    Find the filter used to take an image or turn a given given band into \
    a form that can be used for the calibration.

    Parameters
    ----------
    header : header object
        Header of an image.
    band : str, optional
        if the filter is not in the header it can be given here.
        The default is None.

    Raises
    ------
    ValueError
        No filter found is the header or in given.

    Returns
    -------
    band : string
        band in a form that can be used for the calibration.

    """
    if band is not None:
        band = band
    else:
        try:
            band = header['FILTER']
        except Exception:
            raise ValueError('No filter found nor given')

    u_band = u_sdss()
    g_band = g_sdss() #Different names that can be
    r_band = r_sdss() #found in header
    i_band = i_sdss()
    z_band = z_sdss()
    V_band = V_jc() #Different names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()
    U_band = U_jc()
    clear = c()
    if band in u_band:
        band = 'umag'
    if band in g_band:
        band = 'gmag'
    if band in r_band:
        band = 'rmag'
    if band in i_band:
        band = 'imag'
    if band in z_band:
        band = 'zmag'
    if band in U_band:
        band = 'UMag'
    if band in B_band:
        band = 'BMag'
    if band in V_band:
        band = 'VMag'
    if band in R_band:
        band = 'RMag'
    if band in I_band:
        band = 'IMag'
    if band in clear:
        band = 'cmag'

    return band

### Catalog requests & manipulations of the results ###
def source_coordinates(source_table):
    """
    Give the coordinates of the detected sources in the convenient form for \
    XMatch method.

    Parameters
    ----------
    source_table : astropy.table
        Table containing the informations about detected sources.
        It must contain a sky localisation Column.

    Returns
    -------
    coord_sources : astropy.table
        Table with three columns RA and DEC in degrees and an index.

    """
    coord_sources = Table([source_table['sky_centroid'].ra.deg,
                          source_table['sky_centroid'].dec.deg],
                          names=('_RAJ2000', '_DEJ2000'))
    coord_sources['idx'] = np.arange(len(coord_sources))

    return coord_sources

def xmatch(coordinates, catalog, radius, col_name=['_RAJ2000', '_DEJ2000'] ):
    """
    Perform cross-match with a catalog using the CDS XMatch.

    Parameters
    ----------
    coordinates : astropy.table
        RA, DEC of all detected sources.
    catalog : str
        Vizier identifier of the catalog.
    radius : Float
        radius in arcsecond of the area where to perform the request.

    Returns
    -------
    matched_stars : astropy.table
        The objects in the catalog that matched with the coordinates.

    Notes
    -----
    Vizier catalog identifiers:
    Gaia DR2: I/345/gaia2
    SDSS DR12: V/147/sdss12
    2MASS: II/246/out
    USNO B1: I/284/out
    USNO A2: I/252/out
    GLADE 2: VII/281/glade2
    Panstarrs DR1: II/349/ps1
    Skymaper DR1 : II/358/smss
    """
    matched_stars = XMatch.query(coordinates,
                                 cat2='vizier:%s' % catalog,
                                 max_distance=radius * u.arcsec,
                                 colRA1=col_name[0],
                                 colDec1=col_name[1])

    return matched_stars

def joining(source, matched_star):
    """
    Join a catalog of sources detected in a image and a catalog of objects \
    detected in a reference survey.

    Parameters
    ----------
    source : astropy.table
        Table with the sources detected in an image.
    matched_star : astropy.table
        Table with the objects detected in the survey.

    Returns
    -------
    source : astropy.table
        Join table of the two argument tables.

    """
    flag = np.zeros(len(matched_star))
    matched_star['id'] = np.arange(len(matched_star))
    referenced_star_idx = np.unique(matched_star['idx'])
    closest_id = []
    for idx in referenced_star_idx:
        mask = matched_star['idx'] == idx
        closest_id.append(matched_star[mask]['id'][0])
    # Keep only the closest object in matched_star to a source in the image
    flag[closest_id] = 1
    ref_sources = matched_star[flag == 1]
    source = join(source, ref_sources, keys='idx')

    return source

def cleaning_PS(source):
    """
    Remove the object with bad photometry in a Pan-STARRS table.

    Parameters
    ----------
    source : astropy.table
        Table with the information of PAN-STARRS.

    Returns
    -------
    source : astropy.table
        Table without objects flagged with a poor photometry.

    """
    # Quality for PAN STARRS
    flag = Column(name='Flag Binary',
                  data=np.ones_like(source['Qual'],
                  dtype='a8'))
    source.add_column(flag)
    mask = np.ones_like(source['Qual'], dtype = bool)
    for i, flag in enumerate(source['Qual']):
        bin_flag = bin(source['Qual'][i])[2:].zfill(8)
        source['Flag Binary'][i] = bin_flag
        if bin_flag[-1] == '1' or bin_flag[-3] == '0' or bin_flag[-5] == '0' \
        or bin_flag[-7] == '1' or bin_flag[0] == '1':
        #removing extended objects with first condition
        #removing bad measurement with the second
        #removing objects with bad quality stack
        #removing suspect objects in the stack
        #removing poor quality stack objects
            mask[i] = False
    source = source[mask]

    return source

def cleaning_gaia(source):
    """
    Remove the bad quality photomety objects in the Gaia table returned by
    the crossmatch with the catalog.
    Are considered bad quality objects:
        - duplicated source
    See here for details about the column names :
        https://vizier.u-strasbg.fr/viz-bin/VizieR-3?-source=I/345

    Parameters
    ----------
    source : astropy table
        Table return after a crossmatch with Gaia via the vizier portal.

    Returns
    -------
    source : astropy table
        Input table without the rejected objects.

    """
    source = source[source['duplicated_source']==0]

    return source

def cleaning_sdss(source):
    """
    Remove the bad quality photomety objects in the SDSS table returned by
    the crossmatch with the catalog.
    Are considered bad quality objects:
        - q_mode : indicate the quualitu of the photometry.
                   We keep only the objects with value "+"(good photoometry).
        - Q : Quality of teh observation.
              We keep only objects with value 3 (good observations).
        - class : nature of the object star or galaxy.
                  We keep only objects with value 6 (stars).
        - flag to remove (Hexadecimal values):
            - saturated objects
            - objets on the edge
            - blended objects
            - moving objects

    See here for details about the column names :
        https://vizier.u-strasbg.fr/viz-bin/VizieR-3?-source=V/147

    Parameters
    ----------
    source : astropy table
        Table return after a crossmatch with SDSS DR 12 via the vizier portal.

    Returns
    -------
    source : astropy table
        Input table without the rejected objects.

    """
    cleaning_mask = (source['Q']==3)
    #cleaning_mask &= (source['q_mode']=='+')
    cleaning_mask &= (source['class']==6)
    # pkl = open(path_muphoten +'/sdss_flags_list.pkl', "rb")
    # hexa_flag = pickle.load(pkl) # list with all the flags for the DR12
    hexa_flag = [#(4, 'EDGE'),
                 #(8, 'BLENDED'),
                 (262144, 'SATURATED'),
                 (2147483648, 'MOVED')] # flags to remove
    source['hexa_flags'] = [int('0x' + str(i),16) for i in source['flags']]
    source['to_flags'] = 0
    for i, _ in enumerate(source['hexa_flags']):
        for fid, _ in hexa_flag:
            if i&fid>0:
                source['to_flags'][i]=1
                print(i)
                print(_)
                print(fid)
    cleaning_mask &= (source['to_flags']==0)
    source = source[cleaning_mask]

    return source

def cleaning_usno(source):
    """
    Remove the bad quality photomety objects in the USNO-B1 table returned by
    the crossmatch with the catalog.
    Are considered bad quality objects:
        - Object on a diffraction spike

    See here for details about the column names :
        https://vizier.u-strasbg.fr/viz-bin/VizieR-3?-source=I/284

    Parameters
    ----------
    source : astropy table
        Table return after a crossmatch with USNO-B1 via the vizier portal.

    Returns
    -------
    source : astropy table
        Input table without the rejected objects.

    """
    source['to_flag'] = [1 if 's' in i else 0 for i in source['Flags']]

    return source

def cleaning_crossmatch(cata, phot_table):
    """
    Remove the bad quality objects in the crossmatched table according
    to the chosen catalog.

    Parameters
    ----------
    cata : str
        Vizier index of the catalog used fo rthe calibration.
    phot_table : astropy table
        Table to clean.

    Returns
    -------
    phot_table : astropy table
        Cleaned table.

    """
    if cata=='II/349/ps1':
        phot_table = cleaning_PS(phot_table)
    if cata=='V/147/sdss12':
        phot_table = cleaning_sdss(phot_table)
    if cata=='I/345/gaia2':
        phot_table = cleaning_gaia(phot_table)
    if cata=='I/284/out':
        phot_table = cleaning_usno(phot_table)

    return phot_table

### Catalog requests & manipulations of the results ###
### Functions to compute different photometric transformation laws  ###
def poly(x, coefficients):
    """
    Compute a polynome, useful for transformation laws.

    Parameters
    ----------
    x : list or array like quantity
        Variable or the polynome.
    coefficients : list or array like quantity
        Coefficient of the polynome.

    Returns
    -------
    polys : list or array like quantity
        Result of the polynome computation.

    """
    polys = 0
    for i, coef in enumerate(coefficients):
        polys += coef * x**i

    return polys

def from_gaia2Johnson(band, Gaia_Table):
    """
    Give the transformation laws to go from gaia photometric system \
    to Johnson-Cousins photometric system\
    Transformation given by https://gea.esac.esa.int/archive/documentation/ \
    GDR2/Data_processing/chap_cu5pho/sec_cu5pho_calibr/ \
    ssec_cu5pho_PhotTransf.html \
    Table 5.8 \
    WARNING : GAIA's Magnitudes are in Vega.

    Parameters
    ----------
    band: band used to get the image
    Gaia_Table: astropy.table with information from Gaia for
    the sources in the image

    Returns
    -------
    Results : QTable with informations from Gaia and the computation
             for the band of the image
    """
    V_band = V_jc() #Differen15.690t names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()

    if band in V_band:
        mask = (-0.5 < Gaia_Table['bp_rp'])
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 2.75)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [-0.01760, -0.006860, -0.1732]
        Result["VMag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                            coefficients)
        Result["error"] = 0.045858

    if band in R_band:
        mask = (-0.5 < Gaia_Table['bp_rp'])
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 2.75)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [-0.003226, 0.3833, -0.1345]
        Result["RMag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                            coefficients)
        Result["error"] = 0.04840

    if band in I_band:
        mask = (-0.5 < Gaia_Table['bp_rp'])
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 2.75)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [0.02085, 0.7419, -0.09631]
        Result["IMag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                            coefficients)
        ########Revoir cette derniere transformation, pt etre mal implementee
        Result["error"] = 0.04956

    if band in B_band:
        raise ValueError('No transformation law for Gaia->B_Jonhson')

    return Result

def from_gaia2SDSS(band, Gaia_Table):
    """
    Give the transformation laws to go from gaia photometric system
    to SDSS photometric system
    Transformation given by https://gea.esac.esa.int/archive/documentation/
    GDR2/Data_processing/chap_cu5pho/sec_cu5pho_calibr/
    ssec_cu5pho_PhotTransf.html
    Table 5.7
    WARNING : GAIA's Magnitudes are in the Vega system

    parameters: band: band used to get the image
                Gaia_Table: astropy.table with information from Gaia for
                the sources in the image.

    returns: astropy.table with informations from Gaia and the computation
             for the band of the image
    """
    u_band = u_sdss()
    g_band = g_sdss() #Different names that can be
    r_band = r_sdss() #found in header
    i_band = i_sdss()
    z_band = z_sdss()

    if band in r_band:
        mask = (Gaia_Table['bp_rp'] > 0.2)
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 2.7)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [-0.12879, 0.24662, -0.027464, -0.049465]
        Result["rmag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                          coefficients)
        Result["error"] = 0.066739

    if band in i_band:
        mask = (Gaia_Table['bp_rp'] > 0.0)
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 4.5)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [-0.29676, 0.64728, -0.10141, 0.]
        Result["imag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                            coefficients)
        Result["error"] = 0.098957

    if band in g_band:
        mask = (-0.5 < Gaia_Table['bp_rp'])
        Result = Gaia_Table[mask]
        mask = (Result['bp_rp'] < 2.0)
        Result = Result[mask] #Validity of the transformation law
        coefficients = [0.13518, -0.46245, -0.25171, 0.021349]
        Result["gmag"] = Result["phot_g_mean_mag"] - poly(Result['bp_rp'],
                                                            coefficients)
        Result["error"] = 0.16497

    if band in z_band:
        raise ValueError('No transformation law for Gaia->z_sdss')

    if band in u_band:
        raise ValueError('No transformation law for Gaia->u_sdss')

    return Result

def from_PS2clear(band, PS_Table):

    c_band = c()
    if band in c_band:
        PS_Table['flux_r'] = 3631 * 10**(-0.4 * PS_Table["rmag"])
        PS_Table['flux_g'] = 3631 * 10**(-0.4 * PS_Table["gmag"])
        PS_Table['cmag'] = -2.5 * np.log10((PS_Table['flux_r'] +
                                            PS_Table['flux_g']) / 3631)
        PS_Table['error'] = np.sqrt((PS_Table['flux_r'] / (PS_Table['flux_g'] + 
                                     PS_Table['flux_r']))**2 * PS_Table["e_rmag"]**2 + 
                                    (PS_Table['flux_g'] / (PS_Table['flux_g'] + 
                                     PS_Table['flux_r']))**2 * PS_Table["e_gmag"]**2)

    return PS_Table

def from_skm2clear(band, skm_Table):

    c_band = c()
    if band in c_band:
        skm_Table['flux_r'] = 3631 * 10**(-0.4 * skm_Table["rPSF"])
        skm_Table['flux_g'] = 3631 * 10**(-0.4 * skm_Table["gPSF"])
        skm_Table['cmag'] = -2.5 * np.log10((skm_Table['flux_r'] +
                                             skm_Table['flux_g']) / 3631)
        skm_Table['error'] = np.sqrt((skm_Table['flux_r'] / (skm_Table['flux_g'] + 
                                     skm_Table['flux_r']))**2 * skm_Table["e_rPSF"]**2 + 
                                    (skm_Table['flux_g'] / (skm_Table['flux_g'] + 
                                     skm_Table['flux_r']))**2 * skm_Table["e_gPSF"]**2)

    return skm_Table

def from_PS2Johnson(band, PS_Table):

    V_band = V_jc() #Different names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()

    if band in R_band:
        coefficients = [-0.163, -0.086, -0.061]
        PS_Table['g-r'] = PS_Table['gmag'] - PS_Table['rmag']
        PS_Table["RMag"] = PS_Table["rmag"] + poly(PS_Table['g-r'],
                                                   coefficients)
        PS_Table["error"] = 0.041

    if band in V_band:
        coefficients = [-0.020, -0.498, -0.008]
        PS_Table['g-r'] = PS_Table['gmag'] - PS_Table['rmag']
        PS_Table["VMag"] = PS_Table["gmag"] + poly(PS_Table['g-r'],
                                                   coefficients)
        PS_Table["error"] = 0.032

    if band in B_band:
        coefficients = [0.199, 0.540, 0.016]
        PS_Table['g-r'] = PS_Table['gmag'] - PS_Table['rmag']
        PS_Table["BMag"] = PS_Table["gmag"] + poly(PS_Table['g-r'],
                                                   coefficients)
        PS_Table["error"] = np.sqrt((PS_Table['e_gmag']**2 *\
                                    (1 + coefficients[-2] +
                                     2*coefficients[-1]*PS_Table['g-r'])**2) +
                                    (PS_Table['e_rmag']**2 *\
                                    (2*coefficients[-1]*PS_Table['g-r'] +
                                     coefficients[-2])**2))
        # PS_Table["error_transfo"] = np.sqrt((PS_Table['e_gmag']**2 +
                                              # PS_Table['e_rmag']**2)) *
                                              # (2*coefficients[-1]*
                                              # PS_Table['g-r'] +
                                              # coefficients[-2])
    if band in I_band:
        coefficients = [-0.387, -0.123, -0.03]
        PS_Table['g-r'] = PS_Table['gmag'] - PS_Table['rmag']
        PS_Table["IMag"] = PS_Table["imag"] + poly(PS_Table['g-r'],
                                                   coefficients)
        PS_Table["error"] = 0.054

    return PS_Table

def from_usno2Johnson(band, USNO_Table):
    """
    Give the transformation laws to go from USNO photometric system
    to Johnson photometric system
    Transformation given by http://www.mpe.mpg.de/~jcg/GROND/calibration.html

    parameters: band: band used to get the image
                USNO_Table: astropy.table with information from USNO for
                the sources in the image.

    returns: astropy.table with informations from USNO and the computation
             for the band of the image.
    """
    V_band = V_jc() #Different names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()

    if band in V_band:
        USNO_Table['VMag'] = (0.444 * USNO_Table['B1mag'] +
                              0.556 * USNO_Table['R1mag'])
        USNO_Table["error_from_transformation"] = 0.5
    if band in B_band:
        USNO_Table['BMag'] = USNO_Table['B1mag']
        USNO_Table["error"] = 0.5
    if band in R_band:
        USNO_Table['RMag'] = USNO_Table['R1mag']
        USNO_Table["error"] = 0.5
    if band in I_band:
        USNO_Table['IMag'] = USNO_Table['Imag']
        USNO_Table["error"] = 0.5
#        raise ValueError('No transformation law known for USNO->I_Johnson')

    return USNO_Table

def from_SDSS2Johnson(band, SDSS_Table):
    """
    Give the transformation laws to go from SDSS photometric system
    to Johnson photometric system
    Transformation given by http://www.sdss3.org/dr8/algorithms/
    sdssUBVRITransform.php

    parameters: band: band used to get the image
                SDSS_Table: astropy.table with information from SDSS for
                the sources in the image

    returns: astropy.table with informations from SDSS and the computation
             for the band of the image
    """
    V_band = V_jc() #Different names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()
    U_band = U_jc()

    if band in V_band:
        coefficients = [- 0.016, -0.573]
        SDSS_Table['g-r'] = SDSS_Table['gmag'] - SDSS_Table['rmag']
        SDSS_Table["VMag"] = SDSS_Table["gmag"] + poly(SDSS_Table['g-r'],
                                                            coefficients)
        SDSS_Table["error"] = np.sqrt((0.002 * SDSS_Table['g-r'])**2 +
                                              0.573**2 * SDSS_Table['e_rmag']**2
                                              + 0.427**2 * SDSS_Table['e_gmag']**2
                                              + 0.002**2)

    if band in R_band:
        coefficients = [0.152, -0.257]
        SDSS_Table['r-i'] = SDSS_Table['rmag'] - SDSS_Table['imag']
        SDSS_Table["RMag"] = SDSS_Table["rmag"] + poly(SDSS_Table['r-i'],
                                                            coefficients)
        SDSS_Table["error"] = np.sqrt(0.743**2 *SDSS_Table['e_rmag']**2+
                                              0.257**2 *SDSS_Table['e_imag']**2+
                                              0.002**2 +
                                              SDSS_Table['r-i']**2 * 0.004**2)


    if band in I_band:
        coefficients = [-0.394, -0.409]
        SDSS_Table['i-z'] = SDSS_Table['imag'] - SDSS_Table['zmag']
        SDSS_Table["IMag"] = SDSS_Table["imag"] + poly(SDSS_Table['i-z'],
                                                            coefficients)
        SDSS_Table["error"] = np.sqrt(0.591**2 *SDSS_Table['e_imag']**2+
                                              0.409**2 *SDSS_Table['e_zmag']**2+
                                              0.002**2 +
                                              SDSS_Table['i-z']**2 * 0.006**2)

    if band in B_band:
        coefficients = [0.219, 0.312]
        SDSS_Table['g-r'] = SDSS_Table['gmag'] - SDSS_Table['rmag']
        SDSS_Table["BMag"] = SDSS_Table["gmag"] + poly(SDSS_Table['g-r'],
                                                            coefficients)
        SDSS_Table["error"] = np.sqrt(1.312**2 * SDSS_Table['e_gmag']**2 +
                                              0.312**2 * SDSS_Table['e_rmag']**2 +
                                              0.002**2 +
                                              SDSS_Table['g-r']**2 * 0.003**2)
    if band in U_band:
        raise ValueError('No transformation law for SDSS -> U')

    return SDSS_Table

def change_photometric_system(image_phot_syst, cata_phot_syst,
                              band, phot_table):

    if cata_phot_syst == 'ps':
        if 'jc' in image_phot_syst:
            print('Changing from Pan-STARRS photometric \n'
                  'system to Johnson-Cousins')
            phot_table = from_PS2Johnson(band, phot_table)
        if 'other' in image_phot_syst:
            print('Changing from Pan-STARRS photometric \n'
                  'system to Unfiltered')
            phot_table = from_PS2clear(band, phot_table)
    if cata_phot_syst == 'sdss':
        if 'jc' in image_phot_syst:
            print('Changing from SDSS photometric\
                  system to Johnson-Cousins')
            phot_table = from_SDSS2Johnson(band, phot_table)
        if 'other' in image_phot_syst:
            print('Changing from SDSS photometric \n'
                  'system to Unfiltered')
            phot_table = from_PS2clear(band, phot_table)
    if cata_phot_syst == 'gaia':
        if 'jc' in image_phot_syst:
            print()
            phot_table = from_gaia2Johnson(band, phot_table)
        if ('sdss' or 'ps') in image_phot_syst:
            print('Changing from Gaia photometric \n'
                  'system to SDSS/Pan-STARRS')
            phot_table = from_gaia2SDSS(band, phot_table)
        if 'other' in image_phot_syst:
            raise ValueError('No transformation law for Gaia -> Unfiltered')
    if cata_phot_syst == 'usno':
        if 'jc' in image_phot_syst:
            print('from usno to jc')
            print('Changing from USNO-B1 photometric \n'
                  'system to Johnson-Cousins')
            phot_table = from_usno2Johnson(band, phot_table)

        else:
            raise ValueError('No transformation law\
                             for USNO-B1 -> {}'.format(image_phot_syst[0]))
    if cata_phot_syst == 'skm':
        phot_table = from_skm2clear(band, phot_table)

        return phot_table

def calibration(source, catalog, cata_phot_syst, stars, wcs_data,
                n_sigma=2.0, header=None, cut_mag=21., band=None):

    if band is None:
        band = get_filter(header)
    image_phot_syst = get_photmetric_system(band)
    coord_sources = source_coordinates(source)
    crossmatch = xmatch(coord_sources, catalog, 5.0)
    source = joining(source, crossmatch)

    star = get_source(source, stars, wcs_data)
    source = cleaning_crossmatch(catalog, source)
    if cata_phot_syst not in image_phot_syst:
        change_photometric_system(image_phot_syst,
                                  cata_phot_syst,
                                  band, source)
    else:
        source['error'] = source['e_'+ band]
        star['error'] = star['e_'+ band]
    if cut_mag!=0:
        mask_magnitude = (source[band] < cut_mag)
        source = source[mask_magnitude]
    source.sort('Magnitude')
    source['Delta_Mag'] = source[band] - source['Magnitude']
    clip = sigma_clip(source['Delta_Mag'], sigma=n_sigma, masked=True)
    clip_mask = np.invert(clip.recordmask)
    source = source[clip_mask]

    coefs, V = np.ma.polyfit(source['Magnitude'], source[band],
                             1, cov=True)
    source['Calibrated_Mag'] = pl.polyval(source['Magnitude'],
                                          np.flip(coefs))
    da_2, db_2 = V[0,0], V[1,1]
    if band not in star.columns:
        if cata_phot_syst not in image_phot_syst:
            change_photometric_system(image_phot_syst,
                                      cata_phot_syst,
                                      band, star)
    star['Calibrated_Mag'] = pl.polyval(star['Magnitude'],
                                          np.flip(coefs))
    return source, coefs, da_2, db_2, star

def from_Vega2AB(band, mag_table):
    """
    Give the transformation laws to go from Vega system to AB system
    Transformation given by http://www.astronomy.ohio-state.edu/
    ~martini/usefuldata.html
    Based on Blanton et al. (2007)

    parameters: band: band used to get the image.
                mag_Table: astropy.table with magnitued in Vega system.

    returns: astropy.table with informations from PS and the computation
             for the band of the image.
    """
    V_band = V_jc() #Different names that can be
    R_band = R_jc() #found in header
    I_band = I_jc()
    B_band = B_jc()
    g_band = g_sdss() #Different names that can be
    r_band = r_sdss() #found in header
    i_band = i_sdss()
    z_band = z_sdss()

    #Conversion for Johnson-Cousins photometric System
    if band in V_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.02
    if band in B_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] - 0.09
    if band in R_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.21
    if band in I_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.45

    #Conversion for sdss/Pan-STARRS photometric System
    if band in g_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] - 0.08
    if band in r_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.16
    if band in i_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.37
    if band in z_band:
        mag_table['magnitude_AB'] = mag_table['magnitude'] + 0.54

    return mag_table



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess
import hjson
import numpy as np
from astropy.io import fits
from astropy import wcs
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
from astropy import units as u
from muphoten.registration import registration
from muphoten.ps1_survey import ps1_grid, prepare_PS1_sub
from muphoten.utils import (mkdir_p, rm_p, get_corner_coords)
from muphoten.psfex import psfex
from muphoten.mosaic import create_mosaic
from muphoten.catalog import get_filter
from multiprocessing import Pool


def subtraction(filenames, reference, config, soft="hotpants",
                 method="individual", doMosaic=False, 
                 verbose="NORMAL", outLevel=1, nb_threads=8):
    """Substract a reference image to the input image"""

    imagelist = np.atleast_1d(filenames)
    for ima in imagelist:
        # Create folder with substraction results
        path, filename = os.path.split(ima)
        if path:
            folder = path + "/"
        else:
            folder = ""

        resultDir = folder + "subtraction/"
        # mkdir_p(resultDir)

        # Get coordinates of input image
        im_coords = get_corner_coords(ima)
        print(im_coords)
        hdu = fits.open(ima)[0]
        header = hdu.header

        # Define the reference image
        if reference == "ps1":
            band = get_filter(header)
            if band in ["BMag", "VMag", "cmag", "gmag"]:
                band = "g"
            # elif band == "VMag":
            #     band = "g"
            elif band in ["RMag", "rmag"]:
                band = "r"
            elif band in ["IMag", "imag"]:
                band = "i"
            # elif band == "cmag":
            #     band = "g"
            # band = 'g'
            ps1_cell_table = ps1_grid(im_coords)
            #  Get PS1 files with whom to perform substraction
            subfiles = prepare_PS1_sub(
                ps1_cell_table, band, ima, config, verbose=verbose,
                method=method
            )
            regis_info = registration(
                subfiles, config, resultDir=resultDir, reference=reference,
                verbose=verbose
            )

            if soft == "hotpants":
                subFiles = hotpants(regis_info, config, verbose=verbose,
                                    nb_threads=nb_threads)

            #  create a mosaic of all substracted images when
            # ps1_method=='individual'
            #  Mosaic for substracted files
            if method == "individual" and doMosaic:
                subfiles = np.array(subFiles)
                #  Mosaic for input file
                sublist = [i for i in subfiles[:, 0]]
                outName = os.path.splitext(filename)[0] + "_mosaic"
                create_mosaic(
                    sublist, ima, resultDir, outName, config=config,
                    verbose=verbose
                )
                #  Mosaic for ps1 reference files
                sublist = [i for i in subfiles[:, 1]]
                outName = os.path.splitext(filename)[0] + "_mosaic_ps1"
                create_mosaic(
                    sublist, ima, resultDir, outName, config=config,
                    verbose=verbose
                )
                #  Mosaic for substracted files
                sublist = [i for i in subfiles[:, 2]]
                outName = os.path.splitext(filename)[0] + "_mosaic_sub"
                create_mosaic(
                    sublist, ima, resultDir, outName, config=config,
                    verbose=verbose
                )
                #  Mosaic for mask applied to substracted files
                # Actually there is no need
                
                sublist = [i for i in subfiles[:, 3]]
                outName = os.path.splitext(filename)[0] + "_mosaic_sub_mask"
                create_mosaic(
                    sublist, ima, resultDir, outName, config=config,
                    verbose=verbose
                )

        else:
            subfiles = [[ima, reference, None]]
            regis_info = registration(subfiles, config, resultDir=resultDir)
            subFiles = hotpants(regis_info, config, verbose=verbose,
                                  nb_threads=nb_threads)

    return subFiles


def run_hotpants(hotpants_cmd, hotpants_cmd_file, resmask):
    """Command to execute hotpants. Useful for multiprocessing"""

    os.system("echo %s > %s" % (hotpants_cmd, hotpants_cmd_file))

    os.system(hotpants_cmd)
    
    # Set bad pixel values to 0 and others to 1 for sextractor
    hdulist = fits.open(resmask)
    hdulist[0].data[hdulist[0].data == 0] = 1
    hdulist[0].data[hdulist[0].data != 1] = 0
    hdulist.writeto(resmask, overwrite=True)


def hotpants(regis_info, config, verbose="QUIET", nb_threads=8):
    """Image substraction using hotpants"""

    subfiles = []

    #  Loop over the files
    args = []
    for info in regis_info:
        inim = info["inim"]
        refim = info["refim"]
        maskim = info["mask"]

        path, filename = os.path.split(inim)
        if path:
            folder = path + "/"
        else:
            folder = ""

        resfile = os.path.splitext(inim)[0] + "_sub.fits"
        resmask = os.path.splitext(inim)[0] + "_sub_mask.fits"

        subfiles.append([inim, refim, resfile, resmask])
      
        hotpants_cmd = get_hotpants_cmd(
            inim, refim, maskim, resfile, resmask, info, config, verbose,
            run=1
        )
        hotpants_cmd_file = path + os.path.splitext(filename)[0] + "_hotpants.sh"
        args.append([hotpants_cmd, hotpants_cmd_file, resmask])

    p = Pool(nb_threads)
    p.starmap(run_hotpants, args)
    p.close()
    
    """
        #  Check that substraction performed relatively well
        header = fits.getheader(resfile)
        try:
            X2NRM00 = float(header["X2NRM00"])
            if X2NRM00 > 1:
                flag_bad = True
            else:
                flag_bad = False
        except BaseException:
            flag_bad = True

        flag_bad = False

        if flag_bad:
            print("bad substraction")
            #  try again increasing the sigma of third polynomial for the
            # kernel
            hotpants_cmd = get_hotpants_cmd(
                inim, refim, maskim, resfile, resmask, info, config, verbose,
                run=1
            )

            hotpants_cmd_file = path + os.path.splitext(filename)[0] + "_hotpants.sh"
            os.system("echo %s > %s" % (hotpants_cmd, hotpants_cmd_file))
            os.system(hotpants_cmd)
            header = fits.getheader(resfile)
            try:
                X2NRM00 = float(header["X2NRM00"])
                if X2NRM00 > 1:
                    flag_bad = True
                else:
                    flag_bad = False
            except BaseException:
                flag_bad = True

            if flag_bad:
                print("bad substraction")
                #  Try to increase number of stamps
                hotpants_cmd = get_hotpants_cmd(
                    inim, refim, maskim, resfile, resmask, info, config,
                    verbose, run=1
                )

                hotpants_cmd_file = path + \
                    os.path.splitext(filename)[0] + "_hotpants.sh"
                os.system("echo %s > %s" % (hotpants_cmd, hotpants_cmd_file))
                os.system(hotpants_cmd)
        """


    return subfiles


def get_hotpants_cmd(inim, refim, maskim, resfile, resmask, info,
                     config, verbose, run=1):
    """ Create the hotpants command"""
    if verbose == "QUIET":
        verbosity = 0
    elif verbose == "NORMAL":
        verbosity = 1
    elif verbose == "FULL":
        verbosity = 2

    path, filename = os.path.split(inim)
    if path:
        folder = path + "/"
    else:
        folder = ""

    resfile = os.path.splitext(inim)[0] + "_sub.fits"
    resmask = os.path.splitext(inim)[0] + "_sub_mask.fits"

    if run == 1:
        fname = config["hotpants"]["conf"]
    elif run == 2:
        fname = config["hotpants"]["conf2"]
    elif run == 3:
        fname = config["hotpants"]["conf3"]

    with open(fname) as json_file:
        hotpants_conf = hjson.load(json_file)

    if (
        hotpants_conf["ng"] == "auto"
        or hotpants_conf["r"] == "auto"
        or hotpants_conf["rss"] == "auto"
    ):
        #  Compute PSF FWHM on input and ref images
        FWHM_inim = psfex(inim, config, verbose="QUIET")
        FWHM_refim = psfex(refim, config, verbose="QUIET")

    if hotpants_conf["ng"] == "auto":
        #  transfrom to sigma
        sigma_inim = FWHM_inim[0] / (2 * np.sqrt(2 * np.log(2)))
        sigma_refim = FWHM_refim[0] / (2 * np.sqrt(2 * np.log(2)))
        #  As decribed here https://github.com/acbecker/hotpants
        kernel_match = np.sqrt(sigma_inim ** 2 - sigma_refim ** 2)
        kernel_file = path + os.path.splitext(filename)[0] + "_kernel.dat"
        kernel_txt = "Sigma_ima: %.3f   Sigma_PS1: %.3f    Sigma_kernel: %.3f" % (
            sigma_inim,
            sigma_refim,
            kernel_match,
        )
        os.system("echo %s > %s" % (kernel_txt, kernel_file))
        #  update config file for hotpants
        hotpants_conf["ng"] = "3 6 %.2f 4 %.2f 2 %.2f" % (
            0.5 * kernel_match,
            kernel_match,
            2 * kernel_match,
        )

    if hotpants_conf["r"] == "auto":
        #  seeing*2.5  DECAM, arbitray
        #  seeing*2.5 here: https://arxiv.org/pdf/1608.01006.pdf
        hotpants_conf["r"] = str(int(FWHM_inim[0] * 2.35 * 2.5))
    if hotpants_conf["rss"] == "auto":
        # DECAM: seeing*5
        #  seeing*6 here: https://arxiv.org/pdf/1608.01006.pdf
        hotpants_conf["rss"] = str(int(FWHM_inim[0] * 2.5 * 5))

    # Set min and max acceptable values for input and template images
    # Too simple, need to adapt it in the future
    il = str(info["in_lo"])
    iu = str(info["in_up"])
    tl = str(info["ref_lo"])
    tu = str(info["ref_up"])
    overlap = "%s, %s, %s, %s" % (
        info["XY_lim"][0],
        info["XY_lim"][1],
        info["XY_lim"][2],
        info["XY_lim"][3],
    )

    hotpants_cmd = "hotpants -inim %s -tmplim %s -outim %s -omi %s " % (
        inim,
        refim,
        resfile,
        resmask,
    )
    # hotpants_cmd += '-il %s -iu %s -tl %s -tu %s -gd %s ' % (il, iu, tl,
    #                                                           tu, overlap)
    hotpants_cmd += "-il %s -iu %s -tl %s -tu %s " % (il, iu, tl, tu)
    hotpants_cmd += "-tuk %s -iuk %s " % (tu, iu)
    hotpants_cmd += "-ig %s -tg %s " % (info["gain_in"], info["gain_ref"])
    hotpants_cmd += "-v %s " % verbosity

    if maskim:
        hotpants_cmd += "-tmi %s " % maskim

    # Add params from the hjson conf file
    for key, value in hotpants_conf.items():
        hotpants_cmd += "-%s %s " % (key, value)

    return hotpants_cmd



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

Scripts to perfrom astrometrci calibration.
Can use scamp or astrometry.net.

For astrometry.net You need to install locally astrometry.net and to
download the correct indexes files if needed.

"""

import subprocess
import sys
import os
import numpy as np
from astropy.io import fits
from muphoten.utils import (mv_p, mkdir_p, cp_p, getpath)
import xmltodict


def clean_tmp_files(filename, soft="scamp"):
    """Clean tempory files created by either astrometry.net or scamp"""

    fileroot = os.path.splitext(filename)[0]
    if soft == "astrometrynet":
        os.remove(fileroot[0] + "-indx.xyls")
        os.remove(fileroot[0] + ".axy")
        os.remove(fileroot[0] + ".corr")
        os.remove(fileroot[0] + ".match")
        os.remove(fileroot[0] + ".rdls")
        os.remove(fileroot[0] + ".solved")
        os.remove(fileroot[0] + ".wcs")
        os.remove(fileroot[0] + ".fits")
        os.rename(fileroot[0] + ".new", fileroot[0] + ".fits")
    elif soft == "scamp":
        os.remove("prepscamp.cat")
        os.remove("prepscamp.head")
        os.remove("scamp.xml")


def remove_astro_keywords(header):
    """ Remove from header all keywords related to astrometric solution  """

    #  First remove old keywords related to astrometric calibration
    keywords_to_remove = [
        "CRPIX1",
        "CRPIX2",
        "CRVAL1",
        "CRVAL2",
        "CD1_1",
        "CD1_2",
        "CD2_1",
        "CD2_2",
        "CDELT1",
        "CDELT2",
        "RADESYS",
        "CTYPE1",
        "CTYPE2",
        "EQUINOX",
        "CROTA1",
        "CROTA2",
    ]

    #  List of the first letters for standard astrometric coefficients.
    #  Such as TR, PV, SIA
    # coeff_astro = ['TR', 'SIA', 'A_', 'B_', 'AP_', 'BP_', 'LT', 'PV', 'PC']
    #  For PS1 skycell, though not sure what it means
    coeff_astro = ["PV", "PC"]

    for key, value in header.items():
        for coeff in coeff_astro:
            _len = len(coeff)
            if key[:_len] in [coeff]:
                keywords_to_remove.append(key)

    for keyword in keywords_to_remove:
        if keyword in header:
            del header[keyword]

    return header


def header_from_string(scamphead):
    """Create fits header from scamp .head output file"""

    s = ""
    with open(scamphead) as f:
        for line in f:
            s = s + line + "\n"
    header = fits.Header.fromstring(s, sep="\n")

    return header


def update_headers_scamp(filename, scamphead, pixelscale):
    """Modify the header after running scamp"""

    hdulist = fits.open(filename)
    # Verify and try to fix issue with fits standard
    hdulist.verify("fix")
    hdr = hdulist[0].header
    #  First remove old keywords related to astrometric calibration
    hdr = remove_astro_keywords(hdr)

    newheader = header_from_string(scamphead)

    for key, value in newheader.items():
        # print (key, value)
        # truncate long keys
        if len(key) > 8:
            key = key[:7]
        try:
            hdr.set(key.upper(), value)
        except BaseException:
            try:
                hdr.set(key.upper(), str(value))
            except BaseException:
                pass
    hdr.set("CDELT1", pixelscale[0])
    hdr.set("CDELT2", pixelscale[1])

    if (hdr["CTYPE1"] != "RA---TPV") and (hdr["CTYPE2"] != "DEC--TPV"):
        print("\nWARNING: scamp did not set CTYPE1, CTYPE2 to RA---TPV and DEC--TPV.")
        print(
            "Set them to to these values by hand, otherwise it can not be read by astropy.wcs"
        )
        print(
            "Likely due to some SIP distortions parameters already present in headers."
        )
        print("One might check the astrometry to be safe.\n")
        hdr["CTYPE1"] = "RA---TPV"
        hdr["CTYPE2"] = "DEC--TPV"

    hdulist.writeto(filename, overwrite=True)


def astrometrynet(filename, radius=4, scaleLow=3.5, scaleHigh=4,
                  scaleUnits="arcsecperpix"):
    """Run astrometry.net on the input image"""

    # Get the RA and DEC before removing the keywords as required
    # by astrometry.net.
    # Is it really required? need to check
    hdul = fits.open(filename)
    hdr = hdul[0].header

    ra = str(hdr["CRVAL1"])
    dec = str(hdr["CRVAL2"])

    del hdr["CRVAL1"]
    del hdr["CRVAL2"]
    del hdr["CRPIX1"]
    del hdr["CRPIX2"]
    hdul[0].header = hdr
    hdul.writeto(filename, overwrite=True)
    hdul.close()

    #  Run astrometry.net
    subprocess.call(
        [
            "solve-field", filename,
            "--ra", ra,
            "--dec", dec,
            "--radius", str(radius),
            "--scale-units", str(scaleUnits),
            "--scale-low", str(scaleLow),
            "--scale-high", str(scaleHigh),
            "--crpix-center", "--tweak-order", "5",
            "--uniformize", "0",
            "--no-plots",
            "--overwrite",
        ]
    )

    #  Delete temporary files
    clean_tmp_files(filename, soft="astrometrynet")


def scamp(filename, config, accuracy=0.5, itermax=10,
          band=None, useweight=False, CheckPlot=False,
          verbose="NORMAL"):
    """Compute astrometric solution of astronomical image using scamp"""
    
    path = os.path.dirname(filename)
    imagelist = np.atleast_1d(filename)
    for ima in imagelist:
        print("Performing astrometric calibration on %s using SCAMP." % ima)
        # print ('You required astrometric precision of %.3f arcsec.' % accuracy)

        root = os.path.splitext(ima)[0]
        _name = root.split("/")[-1]

        if CheckPlot:
            plot_fmt = "PNG"
            plotnames = (
                "%s_fgroups,%s_distort,%s_astr_interror2d,%s_astr_interror1d,%s_astr_referror2d,%s_astr_referror1d,%s_astr_chi2,%s_psphot_error"
                % (root, root, root, root, root, root, root, root)
            )
            plottypes = "FGROUPS,DISTORTION,ASTR_INTERROR2D,ASTR_INTERROR1D,ASTR_REFERROR2D,ASTR_REFERROR1D,ASTR_CHI2,PHOT_ERROR"
        else:
            plot_fmt = "NULL"
            plotnames = " "
            plottypes = " "

        if config["telescope"] == "PS1" and band is not None:
            astref_band = band
        else:
            astref_band = "DEFAULT"

        #  Initialise the while loop
        i = 0
        #  Dummy offset
        mean_offset = 100
        while (mean_offset >= accuracy) and (i <= itermax):
            i += 1
            #  Create catalog using sextractor
            # print ('Create FITS-LDAC file from SExtractor')
            print([
                    "sex",
                    "-c", config["scamp"]["sextractor"],
                    "-PARAMETERS_NAME", config["scamp"]["param"],
                    "-VERBOSE_TYPE", verbose,
                    "-FILTER_NAME", config['sextractor']['convFilter'],
                    ima,
                ]
            )
            subprocess.call(
                [
                    "sex",
                    "-c", config["scamp"]["sextractor"],
                    "-PARAMETERS_NAME", config["scamp"]["param"],
                    "-VERBOSE_TYPE", verbose,
                    "-FILTER_NAME", config['sextractor']['convFilter'],
                    ima,
                ]
            )
            #  Run SCAMP
            print([
                    "scamp",
                    "prepscamp.cat",
                    "-c", config["scamp"]["conf"],
                    "-ASTREF_BAND", astref_band,
                    "-CHECKPLOT_DEV", plot_fmt,
                    "-CHECKPLOT_NAME", plotnames,
                    "-CHECKPLOT_TYPE", plottypes,
                    "-VERBOSE_TYPE", verbose,
                ])
            subprocess.call(
                [
                    "scamp",
                    "prepscamp.cat",
                    "-c", config["scamp"]["conf"],
                    "-WRITE_XML", "y",
                    "-ASTREF_BAND", astref_band,
                    "-CHECKPLOT_DEV", plot_fmt,
                    "-CHECKPLOT_NAME", plotnames,
                    "-CHECKPLOT_TYPE", plottypes,
                    "-VERBOSE_TYPE", verbose,
                ]
            )
            #  Check astrometry offset
            with open("scamp.xml") as fd:
                doc = xmltodict.parse(fd.read())
            """
            offset = doc['VOTABLE']['RESOURCE']['RESOURCE']['TABLE'][0]['DATA']['TABLEDATA']['TR']['TD'][34]
            offset = offset.split(' ')
            offset_axis1 = float(offset[0])
            offset_axis2 = float(offset[1])
            """
            header = header_from_string("prepscamp.head")
            offset_axis1 = float(header["ASTRRMS1"] * 3600)
            offset_axis2 = float(header["ASTRRMS2"] * 3600)

            mean_offset = np.mean([offset_axis1, offset_axis2])
            print(
                "Astrometric precision after run %d: %.2f arcseconds. Required: %.2f."
                % (i, mean_offset, accuracy)
            )

            pixelscale = doc["VOTABLE"]["RESOURCE"]["RESOURCE"]["TABLE"][0]["DATA"][
                "TABLEDATA"
            ]["TR"]["TD"][18].split("  ")
            pixelscale = [float(pixelscale[0]) / 3600,
                          float(pixelscale[1]) / 3600]
            # Update header of input fits file
            update_headers_scamp(ima, "prepscamp.head", pixelscale)
        # cp_p('prepscamp.cat', _name.split('.')[0]+'.cat')
        #  Delete temporary files
        clean_tmp_files(ima, soft="scamp")
    print("\n")


def astrometric_calib(filenames, config, soft="scamp",
                      accuracy=0.5, itermax=10, verbose="NORMAL"):
    """perform astrometric calibration"""

    imagelist = np.atleast_1d(filenames)
    for ima in imagelist:
        #  Use scamp for astrometric calibration
        if soft == "scamp":
            scamp(ima, config, accuracy=accuracy,
                  itermax=itermax, verbose=verbose)

        #  Use astrometry.net for astrometric calibration
        elif soft == "astrometrynet":
            # Get pixel scale in degrees
            header = fits.getheader(ima)
            try:
                pixScale = abs(header["CDELT1"])
            except Exception:
                try:
                    pixScale = abs(header["CD1_1"])
                except Exception:
                    print("Pixel scale could not be found in fits header.\n"
                          "Expected keyword: CDELT1, _DELT1 or CD1_1")
            #  Set up boundaries for plate scale for astrometry.net
            scaleLow = 0.7 * pixScale * 3600
            scaleHigh = 1.3 * pixScale * 3600
            radius = max(header["NAXIS1"] * pixScale,
                         header["NAXIS2"] * pixScale)
            astrometrynet(ima, radius=radius, scaleLow=scaleLow,
                         scaleHigh=scaleHigh)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Mon Apr 19 16:58:12 2021

@author: duverne, IJClab, Orsay, duverne@protonmail.com
"""

import argparse, os, glob, sys, hjson, warnings
# import time
from astropy.io import fits
from astropy.wcs import WCS, FITSFixedWarning
from astropy import units as u
import numpy as np
import numpy.polynomial.polynomial as pl

from muphoten.background import sub_background
from muphoten.utils import (load_coord_file,
                            getTel, make_results_dir, get_filename,
                            list_files, load_config, set_results_table,
                            make_sub_result_rep, PhotList)
from muphoten.sanitise import sanitise_fits
from muphoten.catalog import (get_filter, getCata, CataList, calibration,
                              catalog_choice, )
from muphoten.photometry import (source_detection, Kron_Photometry, masking,
                                 Fixed_Photometry, Isophotal_Photometry,
                                 Dophot, get_source, compute_error,
                                 compute_snr)
from muphoten.psfex import psfex
from muphoten.plot_image import (plot_sementation_image,
                                 plot_image, plot_calib)

warnings.filterwarnings("ignore", category=FITSFixedWarning)
warnings.simplefilter(action="ignore", category=FutureWarning)

"""Muphoten main script for photometry."""

def main():
    """Console script for muphoten."""
    # path_muphoten = getpath()
    telescope_list = getTel()
    catalog_list = CataList()
    phot_list = PhotList()
    parser = argparse.ArgumentParser(description="Performing photometry of\
                                     transient objects.")

    parser.add_argument("--coord",
                        dest="coord",
                        required=True,
                        type=str,
                        help="Coordinates file's path.")

    parser.add_argument("--images",
                        dest="images",
                        required=True,
                        type=str,
                        help="Path to images.")

    parser.add_argument("--outname",
                        required=True,
                        type=str,
                        help="Name of the output file.")

    parser.add_argument("--telescope",
                        dest="telescope",
                        choices=telescope_list,
                        required=True,
                        type=str,
                        help="Telescope that acquired the images.")

    parser.add_argument("--catalog",
                        dest="catalog",
                        choices=catalog_list,
                        required=False,
                        type=str,
                        help="Catalog to use for the calibration.")

    parser.add_argument("--photo",
                        dest="photo",
                        default='iso',
                        choices=phot_list,
                        help="Type of photometry. Use muphoten configuration \
                              file to modify the radius of the isophote or the \
                              factor used for the fixed apertures.\
                              Default is iso.")

    parser.add_argument("--sub-method",
                        default='sub',
                        choices=['sub', 'nosub'], # , 'nosub'],
                        help="Wether HOTPANTS has been used to perform"
                             "template subtraction (sub) or not (nosub).")

    parser.add_argument("--keep-old",
                        "--keep",
                        dest="keep",
                        required=False,
                        action="store_true",
                        help="Keep previous results")

    parser.add_argument("--skip-processed",
                        "--skip",
                        dest="skip",
                        required=False,
                        action="store_true",
                        help="Skip already processed files")

    parser.add_argument("--band",
                        required=False,
                        choices=['UMag', 'BMag', 'VMag', 'RMag', 'IMag',
                               'umag', 'gmag', 'rmag', 'imag', 'zmag',
                               'cmag'],
                        help="Band used to acquire the image. To use if the"
                             "'FILTER' keyword is not in the fits header."
                             "Use Capital name for Johson-Cousins filters,"
                             "use lowercase names for the Sloan filters,"
                             "use cmag name for unfiltered images and "
                             "luminance filter.")

    parser.add_argument("--mask-size",
                        dest="mask",
                        required=False,
                        type=int,
                        help="Using a square mask centered on the transient."
                             "Beyond this mask, the source are"
                             "not used for the calibration. ")

    parser.add_argument("--save-bkg",
                        dest="save_bkg",
                        action="store_true",
                        help="Save the background images as fits and png.\
                              Default : False")

    args = parser.parse_args()

    # Loading the coordinates of the analysed objects
    sources, stars = load_coord_file(args.coord)
    print('Computing lightcurve for {} sources'.format(len(sources)))
    print('Using {} stars to veto poor quality images'.format(len(stars)))

    print('Loading configuration for Muphoten,')
    config = load_config(args.telescope)
    mu = config['muphoten']['conf']
    with open(mu) as json_file:
        mu_conf = hjson.load(json_file)

    # Preparing the files for analysis
    images = list_files(args.images, get_subdirs=False)
    print('{} images to calibrate'.format(len(images)))
    # glob_start = time.time()
    results_file = os.path.join(args.images, 'results')
    col_name = ['filename',
                'telescope',
                'filter',
                'time',
                'psf',
                'a', 'da',
                'b', 'db',
                'src_centroid_ra',
                'src_centroid_dec',
                'src_flux',
                'src_bkg_flux',
                'src_ins_mag',
                'src_mag',
                'src_SNR',
                'src_bkg_err',
                'src_flux_err',
                'src_calib_err',
                'src_tot_err',
                'src_det',
                'ref_mag',
                'ref_mag_cata',
                'ref_tot_err',
                'ref_cata_err',
                'ref_flux',
                'ref_bkg_flux',
                'ref_SNR',
                'ref_dist']
    res = set_results_table(col_name, len(images))

    for i, image in enumerate(images):
        image = make_results_dir(image,
                                 directory='calibration',
                                 keep=args.keep,
                                 skip=args.skip)
        image_name = get_filename(image)
        res['filename'][i] = image_name
        image_rep = os.path.dirname(image)
        calib_rep = os.path.join(image_rep, 'calibration')
        print(calib_rep)
        sub_rep = os.path.join(image_rep, 'subtraction/')
        print(sub_rep)
        # start_im_time = time.time()  # Computing time for one image processing
        print('Processing image n° {} : {}'.format(i, image_name))
        print('Sanitising header')
        sanitise_fits(image)
        print(image)
        # Load the data
        hdu = fits.open(image)[0]
        data, header = hdu.data, hdu.header
        wcs_data = WCS(header)
        if 'TELESCOP' in header.keys():
            res['telescope'][i] = header['TELESCOP']
        else:
            res['telescope'][i] = args.telescope
        exp_time = float(header['EXPTIME'])

        # Getting different useful informations : band, MJD,
        # catalog for calibration.
        res['time'][i] = header.get('MJD-OBS')
        if args.catalog:
            catalog, system = getCata(args.catalog)
            cata_name = args.catalog
        else:
            catalog, system, cata_name = catalog_choice(header)

        if args.band:
            band = args.band
        else:
            band = get_filter(header)

        res['filter'][i] = band
        
        print('Subtracting background in image : {}'.format(image_name))
        # Getting the backgroud, rms and bkg subtracted images
        if args.save_bkg:
            make_sub_result_rep(image_rep, directory='background')
            path2save = image_rep
        else:
            path2save = None
        data_clean, background,\
            rms = sub_background(data, header,
                                 image_name,
                                 n_sigma=mu_conf['bkg_n_sigma'],
                                 box_size=mu_conf['bkg_box'],
                                 bkg_estimator=mu_conf['bkg_estimator'],
                                 filter_size=mu_conf['bkg_filter_size'],
                                 path2save=path2save,
                                 png=args.save_bkg)

        # Detection of the sources & segmentation
        mask=None
        if args.mask:
            mask = masking(data_clean, sources, header, args.mask)
        segmentation = source_detection(data, background, rms,
                                        n_sigma=mu_conf['det_n_sigma'],
                                        deblend=mu_conf['det_deblend'],
                                        mask=mask,
                                        threshold_level=mu_conf['det_treshold'],
                                        npixels=mu_conf['det_npixels'],
                                        contrast=mu_conf['contrast'])
        plot_sementation_image(segmentation, path2save=calib_rep)
        # Computing PSF
        print('Computing PSF for image n° {}'.format(i))
        make_sub_result_rep(image_rep, directory='psf')
        FWHM_psf = psfex(image, config, verbose="QUIET")
        res['psf'][i] = FWHM_psf[0]
        print(FWHM_psf[0])
        if args.photo == phot_list[0]:  # Kron photometry
            print('Performing Kron Photometry')
            photometry_table = Kron_Photometry(data, data_clean, segmentation,
                                               header, background, rms,
                                               wcs=wcs_data)
            plot_image(data_clean, ap=photometry_table['kron_aperture'],
                       wcs=wcs_data, path2save=calib_rep)
        if args.photo == phot_list[1]:
            print('Performing Isophotal Photometry')
            photometry_table = Isophotal_Photometry(data, data_clean,
                                                    segmentation, header,
                                                    background, rms,
                                                    radius=mu_conf['radius_iso'],
                                                    wcs=wcs_data)
            plot_image(data_clean, sky_ap=photometry_table['aperture'],
                       wcs=wcs_data, path2save=calib_rep)
        if args.photo == phot_list[2]:
            print('Performing Fixed size Photometry')
            radius = mu_conf['fixed_factor'] * FWHM_psf[0]
            photometry_table = Fixed_Photometry(data, data_clean, segmentation,
                                                header, radius, background,
                                                rms, wcs=wcs_data)
            plot_image(data_clean, sky_ap=photometry_table['aperture'],
                       wcs=wcs_data, path2save=calib_rep)

        if args.sub_method == 'nosub': # no template subtraction performed
            src = get_source(photometry_table, sources, wcs_data)
            dist = src['dist'][0]

        print('Performing calibration using {}'.format(cata_name))
        photometry_table, coefs,\
            da_2, db_2, star_ref = calibration(source=photometry_table,
                                               catalog=catalog,
                                               cata_phot_syst=system,
                                               stars=stars,
                                               wcs_data=wcs_data,
                                               n_sigma=mu_conf['clipping_calib'],
                                               cut_mag=mu_conf['cut_mag'],
                                               band=band)
        plot_calib(band, photometry_table,
                   coefs, da_2, db_2,
                   path2save=calib_rep)
        photometry_table.write(calib_rep + '/calibration.dat',
                               format='ascii.commented_header',
                               overwrite=True)

        # Saving the calibration results.
        res['a'][i] = coefs[0]
        res['da'][i] = np.sqrt(da_2)
        res['b'][i] = coefs[1]
        res['db'][i] = np.sqrt(db_2)

        # Saving the results for the reference star.
        star_ref = compute_error(star_ref, coefs[0], da_2, db_2)
        star_ref = compute_snr(star_ref)
        star_ref = star_ref.filled(0.) # fill the table empty columns with zero
                                       # to avoid issue with masked value.

        res['ref_tot_err'][i] = star_ref['total_error'][0]
        res['ref_mag_cata'][i] = star_ref[band][0]
        res['ref_cata_err'][i] = star_ref['error'][0]
        res['ref_flux'][i] = star_ref['aperture_sum'][0]
        res['ref_mag'][i] = star_ref['Calibrated_Mag'][0]
        res['ref_bkg_flux'][i] = star_ref['background_sum'][0]
        res['ref_SNR'][i] = star_ref['SNR'][0]

        if star_ref['dist'][0] < 5:
            res['ref_dist'][i] = 0  # Reference star detected
        else:
            res['ref_dist'][i] = 1  # Reference star not detected

        if args.sub_method != 'nosub': # no template subtraction performed
            # Detecting the transient in the subtracted image.
            print('Treating subtracted image')
            sub_im = glob.glob(sub_rep + '*.fits')[0]
            print('SUBIMAGE =', sub_im)
            hdu_sub = fits.open(sub_im)[0]
            data_sub, header_sub = hdu_sub.data, hdu_sub.header
            wcs_sub = WCS(header_sub)

            # start_sub_time = time.time()
            mask = (data_sub == 1e-30) | (data_sub == 0)
            data_clean_sub, background_sub,\
                rms_sub = sub_background(data_sub, header_sub,
                                         image_name,
                                         n_sigma=mu_conf['bkg_n_sigma_sub'],
                                         box_size=mu_conf['bkg_box_sub'],
                                         bkg_estimator=mu_conf['bkg_estimator'],
                                         filter_size=mu_conf['bkg_filter_size_sub'],
                                         mask=mask)
            # Creating a mask centered on the transient to avoid the source
            # detection in all the sub image. Otherwise, there could issues
            # because of artefacts due to the subtraction in the sub image.
            mask = masking(data_clean_sub, sources, header_sub, 50)
            seg_sub = source_detection(data_sub,
                                       background_sub,
                                       rms_sub,
                                       n_sigma=mu_conf['det_n_sigma_sub'],
                                       deblend=mu_conf['det_deblend_sub'],
                                       threshold_level=mu_conf['det_treshold_sub'],
                                       npixels=mu_conf['det_npixels_sub'],
                                       contrast=mu_conf['contrast_sub'],
                                       mask=mask)
            # If the transient is not detected in the image
            if seg_sub is None:
                res['src_det'][i] = 1
                print('No transient detected')
                continue
            else:
                if args.photo == phot_list[0]:  # Kron photometry
                    print('Performing Kron Photometry in sub image')
                    photometry_sub = Kron_Photometry(data_sub, data_clean_sub,
                                                     seg_sub,
                                                     header_sub, background_sub,
                                                     rms_sub, wcs=wcs_sub)
                if args.photo == phot_list[1]:
                    print('Performing Isophotal Photometry in sub image')
                    photometry_sub = Isophotal_Photometry(data_sub,
                                                          data_clean_sub,
                                                          seg_sub, header_sub,
                                                          background_sub,
                                                          rms_sub,
                                                          radius=mu_conf['radius_iso'],
                                                          wcs=wcs_sub)

                if args.photo == phot_list[2]:
                    print('Performing Fixed size Photometry in sub image')
                    radius = mu_conf['fixed_factor'] * FWHM_psf[0]
                    photometry_sub = Fixed_Photometry(data_sub,
                                                      data_clean_sub,
                                                      seg_sub,
                                                      header_sub, radius,
                                                      background_sub,
                                                      rms_sub, wcs=wcs_sub)
    
                src = get_source(photometry_sub, sources, wcs_sub)
                print('SOURCE', len(src))
                dist = src['dist'][0]
                if args.photo == phot_list[0]:
                    plot_image(data_clean_sub,
                               ap=src['kron_aperture'],
                               wcs=wcs_sub,
                               path2save=calib_rep,
                               name='/sub_aperture')
                else:
                    plot_image(data_clean_sub,
                               sky_ap=src['aperture'],
                               wcs=wcs_sub, path2save=calib_rep,
                               name='/sub_aperture')
        print('Distance to the source = {}'.format(dist))
        if dist < 15.: # Considering that the transient has been detected
            res['src_det'][i] = 0
            flux = src['aperture_sum'][0]
            if args.sub_method=='sub':
                flux = flux * exp_time

            mag = -2.5*np.log10(flux)
            bkg_sum = Dophot(background, sources, wcs_data,
                             aperture=src['aperture'][0])

        else:
            res['src_det'][i] = 1
            # Computing the number of counts in a 5 pixels circular aperture
            # to have an upper limit on the transient's magnitude.
            if args.sub_method == 'nosub':
                flux = Dophot(data_clean, sources, wcs_data, radius=5.0)
            # elif args.sub_method=='ps':
            #     flux = Dophot(data_sub, sources, wcs_sub, radius=5.0) * exp_time
            else:
                flux = Dophot(data_sub, sources, wcs_sub, radius=5.0) * exp_time
            mag = -2.5*np.log10(flux)
            bkg_sum = Dophot(background, sources, wcs_data, radius=5.0)
        res['src_centroid_ra'][i] = float(src['sky_centroid'].ra / u.deg)
        res['src_centroid_dec'][i] = float(src['sky_centroid'].dec / u.deg)
        res['src_flux'][i] = abs(flux)
        res['src_bkg_flux'][i] = bkg_sum
        res['src_ins_mag'][i] = mag
        res['src_mag'][i] = pl.polyval(mag, np.flip(coefs))
        res['src_SNR'][i] = flux / np.sqrt(flux + bkg_sum)
        res['src_bkg_err'][i] = 2.5 / (np.log(10)*np.sqrt(abs(bkg_sum)))
        res['src_flux_err'][i] = 2.5/(np.log(10)*np.sqrt(flux))
        res['src_calib_err'][i] = np.sqrt(mag**2 * da_2 + db_2 +
                                          coefs[0]*res['src_flux_err'][i]**2)
        res['src_tot_err'][i] = np.sqrt(res['src_flux_err'][i]**2 +
                                            res['src_bkg_err'][i]**2 +
                                            res['src_calib_err'][i]**2)
        print('Transient detected with magnitude {}'.format(res['src_mag'][i]))
        print('Error on the magnitude is {}'.format(res['src_tot_err'][i]))

        # res.show_in_browser(jsviewer=True)
    # res.show_in_browser(jsviewer=True)
    res.write(results_file + '/' + args.outname,
              format='ascii.commented_header',
              overwrite=True)
    return 0


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Created on Wed Feb 10 18:02:32 2021

@author: duverne, corre
"""

import argparse
import warnings
import glob, os

from astropy.io import fits

from muphoten.utils import (load_config, list_files, 
                            make_results_dir,
                            clean_outputs, getpath, getTel, get_filename)
from muphoten.sanitise import sanitise_fits
from muphoten.subtraction import subtraction
from muphoten.astrometry import astrometric_calib
from muphoten.plot_image import plot_image

# clean_folder, cut_image
warnings.simplefilter(action="ignore", category=FutureWarning)

"""Muphoten script for sutracting template image with HOTPANTS."""

def main():

    path_muphoten = getpath()

    telescope_list = getTel()

    parser = argparse.ArgumentParser(
        usage="usage: %(prog)s data [data2 ... dataN] [options]",
        description="Subtraction with HOTPANTS in astronomical images.")

    parser.add_argument(
        "--images",
        required=True,
        type=str,
        help="Base path for the images to analyse.")

    parser.add_argument(
        "--skip-processed",
        "--skip",
        dest="skip",
        required=False,
        action="store_true",
        help="Skip already processed files")

    parser.add_argument(
        "--keep-processed",
        "--keep",
        dest="keep",
        required=False,
        action="store_true",
        help="Skip already processed files")

    parser.add_argument(
        "--telescope",
        dest="telescope",
        choices=telescope_list,
        required=True,
        type=str,
        help="Alias for the available telescopes.")

    parser.add_argument(
        "--astrometry",
        dest="doAstrometry",
        required=False,
        default="scamp",
        choices=["no", "scamp"],
        help="Whether to perform astrometric calibration, with scamp. ")
    
    parser.add_argument(
        "--sub",
        dest="doSub",
        # choices=["ps1", "path to reference image"],
        default="ps1",
        required=False,
        type=str,
        help="Whether to perform astrometric calibration, with ps1 images "
             'or user provided reference image. Type "ps1" for PS1 reference '
             'image or provide the path to your reference image.')

    parser.add_argument(
        "--ps1-method",
        dest="ps1_method",
        required=False,
        default="mosaic",
        choices=["mosaic", "individual"],
        type=str,
        help="When substracting images using Pan-STARRS reference images, "
             "there 2 options, either create a mosaic of all PS1 image and "
             "substract or do the substraction individually for each PS1 "
             "image. In the latter case, your image is cut to match the "
             "PS1 image. (Default: mosaic)")

    parser.add_argument(
        "--mosaic",
        dest="doMosaic",
        action="store_true",
        help="Whether to combine the individual frames into a common mosaic "
             "when `ps1_method` is set to `individual`. (Default: not set)")

    parser.add_argument(
        "--output-data-level",
        dest="outLevel",
        required=False,
        type=int,
        default=0,
        choices=[0, 1, 2],
        help="Number of output files that are kept after the process. "
             "0: minimum, 2: maximum"
             "(Default: 0)")

    parser.add_argument(
        "--conv-filter",
        dest="convFilter",
        required=False,
        default="default",
        type=str,
        help="Corresponds to FILTER_NAME keyword for sextractor "
             "(without .conv)."
             "\nDifferent filter available listed here: %s"
        % path_muphoten + "/config/conv_kernels/"
             "\n(Default: default)")

    parser.add_argument(
        "--verbose",
        dest="verbose",
        required=False,
        default="NORMAL",
        choices=["QUIET", "NORMAL", "FULL", "LOG"],
        type=str,
        help="Level of verbose, according to astromatic software. ")
    
    # parser.add_argument(
    #     "--quadrants",
    #     dest="quadrants",
    #     required=False,
    #     default=1,
    #     type=int,
    #     help="Number of quadrants the image is divided. "
    #          "(Default: 1)")

    args, filenames = parser.parse_known_args()

    # Nb_cuts = (args.quadrants, args.quadrants)

    # Load config files for a given telescope
    print('Loading configuration')
    config = load_config(args.telescope, args.convFilter)

    print('Looking for the images')
    images = list_files(args.images, get_subdirs=False)
    print(images)

    if (args.doAstrometry != "no") and (args.doSub != "ps1"):
        astrometric_calib(args.doSub,
                          config,
                          soft=args.doAstrometry,
                          verbose=args.verbose,
                          accuracy=0.15,
                          itermax=10)

    for i, image in enumerate(images):
        print(image)
        image = make_results_dir(image,
                                 directory='subtraction',
                                 keep=args.keep,
                                 skip=args.skip)
        image_name = get_filename(image)

        if not image:
            print("{} is already processed, skipping. \n".format(image_name))
            continue

        print("Sanitise header and data of {}.\n".format(image_name))
        sanitise_fits(image)

        # # Cut image into several quadrants if required
        # # And create table with filename and quadrant ID
        # image_table = cut_image(
        #     filename,
        #     config,
        #     Nb_cuts=Nb_cuts,
        #     doAstrometry=args.doAstrometry
        # )

        if args.doAstrometry != "no":
            astrometric_calib(image,
                              config,
                              soft=args.doAstrometry,
                              verbose=args.verbose,
                              accuracy=0.15,
                              itermax=10)

        subtraction(image,
                     args.doSub,
                     config,
                     soft="hotpants",
                     method=args.ps1_method,
                     doMosaic=args.doMosaic,
                     verbose=args.verbose,
                     outLevel=args.outLevel,
                     nb_threads=8)
        # clean output files
        clean_outputs(image, args.outLevel)

        image_rep = os.path.dirname(image)
        sub_rep = os.path.join(image_rep, 'subtraction')
        sub = glob.glob(sub_rep + '/*.fit*')
        hdu = fits.open(sub[0])[0]
        data = hdu.data
        plot_image(data, path2save=sub_rep, name='/sub_im')


if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Author: David Corre
# email: corre@lal.in2p3.fr

import shutil, subprocess
import numpy as np
from astropy.io import fits
from muphoten.astrometry import scamp
from muphoten.utils import rm_p

def create_mosaic(file_list, inputimage, outputDir, outName,
                  config, useweight=False, verbose='NORMAL',
                  astrometry=False):
    """Create a mosaic of images"""

    np.savetxt('mosaic.list', file_list, fmt='%s')

    mosaic_name = outputDir + outName 

    # Get pixel scale from input image header
    header = fits.getheader(inputimage)

    try:
        pixScale = abs(header['CDELT1'])
    except Exception:
        try:
            pixScale = abs(header['CD1_1'])
        except Exception:
            print ('Pixel scale could not be found in fits header.\n'
                   'Expected keyword: CDELT1 or CD1_1')
    pixScale = pixScale * 3600
    #print (inputimage, pixScale)
    crval1 = header['CRVAL1']
    crval2 = header['CRVAL2']
    #print (crval1, crval2)
    #print (header['CRPIX1'], header['CRPIX2'])
    imagesize = [header['NAXIS1'], header['NAXIS2']]

    # Force reference pixel to be in the center

    # File name to store the common header that will be shared by all
    # images in filelist
    point = 'registration'
    # Delete if already exists
    rm_p(point + '.head')
    # First run swarp to create a .head file containing the shared header
    subprocess.call(['swarp', '-HEADER_ONLY', 'Y',
                     '-IMAGEOUT_NAME', point + '.head',
                     '-VERBOSE_TYPE', verbose] + [inputimage])
    # Some keywords manipulation using sed
    subprocess.call(['sed', '-i', \
                             's/MJD-OBS/COMMENT/; s/EXPTIME/COMMENT/; s/GAIN  \
                                 /COMMENT/; s/SATURATE/COMMENT /', \
                     point + '.head'])

    imalists=['@' + 'mosaic.list']
    # Remove mosaic if already exists
    rm_p(mosaic_name + '.fits')
    if 'mask' in mosaic_name:
        subBackground = 'N'
    else:
        subBackground = 'Y'

    # Copy the common header in the .head file
    # So that it is read by sawrp for each image
    shutil.copy(point + '.head', mosaic_name + '.head')

    if useweight:
        subprocess.call(['swarp',
                         '-IMAGEOUT_NAME', mosaic_name + '.fits', \
                         '-WEIGHTOUT_NAME', mosaic_name + '.weight.fits', \
                         '-VERBOSE_TYPE', verbose] + imalists)
    else:
        subprocess.call(['swarp',
                         '-IMAGEOUT_NAME', mosaic_name + '.fits',\
                         '-SUBTRACT_BACK', subBackground, \
                         '-COMBINE', 'Y', \
                         '-BACK_SIZE', '128', \
                         '-BACK_FILTERSIZE', '3',\
                         #'-CENTER_TYPE', 'MANUAL', \
                         #'-CENTER', '%s, %s' % (crval1,crval2), \
                         '-RESAMPLE', 'Y',\
                         '-RESAMPLING_TYPE', 'LANCZOS3',\
                         #'-RESAMPLING_TYPE', 'BILINEAR',\
                         '-PIXELSCALE_TYPE', 'MANUAL', \
                         '-PIXEL_SCALE', str(pixScale), \
                         #'-IMAGE_SIZE', '%s, %s' % (imagesize[0], imagesize[1]), \
                         '-OVERSAMPLING', '0',\
                         '-COMBINE_TYPE', 'MEDIAN', \
                         '-COPY_KEYWORDS', ' PIXEL_SCALE', \
                         '-VERBOSE_TYPE', verbose] + imalists)
    rm_p(mosaic_name+'.head')
    # Perform astrometric calibration of the mosaic with scamp
    if astrometry:
        scamp(mosaic_name + '.fits', config, useweight=False,
              CheckPlot=False, verbose=verbose)

    rm_p('mosaic.list')
    rm_p('swarp.xml')
    rm_p(point+'.head')

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed May  5 23:57:35 2021

@author: David Corre, IJClab, Orsay, duverne@protonmail.com
"""

"""Run psfex to compute PSF."""

import os
import subprocess
import numpy as np
from astropy.io import fits
from muphoten.utils import (rm_p, mv_p, get_filename)
import xmltodict


def psfex(filename, config, useweight=False,
          verbose="NORMAL", outLevel=0, outDir='', save=False):
    """Compute PSF in astronomical images"""

    FWHM_list = []

    # imagelist=glob.glob(path+'/*.fits')
    imagelist = np.atleast_1d(filename)
    for ima in imagelist:
        print("\nRunning psfex to estimate FWHM in %s" % ima)
        root = os.path.join(os.path.dirname(filename), 'psf/')
        name = get_filename(filename)
        if useweight:
            weight = root + ".weight.fits"
            subprocess.call(
                [
                    "sex",
                    ima,
                    "-c", config["psfex"]["sextractor"],
                    "-WEIGHT_IMAGE", weight,
                    "-VERBOSE_TYPE", verbose,
                    "-PARAMETERS_NAME", config["psfex"]["param"],
                    "-FILTER_NAME", config['sextractor']['convFilter'],
                ]
            )
        else:
            subprocess.call(
                [
                    "sex",
                    ima,
                    "-c", config["psfex"]["sextractor"],
                    "-VERBOSE_TYPE", verbose, 
                    "-PARAMETERS_NAME", config["psfex"]["param"],
                    "-FILTER_NAME", config['sextractor']['convFilter'],
                ]
            )
        cat = "preppsfex.cat"
        subprocess.call(
            ["psfex", cat, "-c", config["psfex"]["conf"],
             "-VERBOSE_TYPE", verbose]
        )
        rm_p(cat)

        #  Delete files depending on the required level of output files
        mv_p("snap_preppsfex.fits", root + name + "_psf.fits")

        # Get the mean PSF FWHM in pixels
        with open("psfex.xml") as fd:
            doc = xmltodict.parse(fd.read())
            FWHM_stats = doc["VOTABLE"]["RESOURCE"]["RESOURCE"]["TABLE"][0]["DATA"][
                "TABLEDATA"
            ]["TR"]["TD"][20:23]
            FHWM_min = float(FWHM_stats[0])
            FHWM_mean = float(FWHM_stats[1])
            FHWM_max = float(FWHM_stats[2])

            print("\nFWHM min: %.2f pixels" % FHWM_min)
            print("FWHM mean: %.2f pixels" % FHWM_mean)
            print("FWHM max: %.2f pixels\n" % FHWM_max)

        #  Get number of psf snapshot per axis
        psf_snaps = os.popen(
            "sed -n '/PSFVAR_NSNAP/p' %s" % config["psfex"]["conf"]
        ).read()
        nb_snaps = psf_snaps.split()[1]
        if save:
            # Add info to the header
            hdulist = fits.open(root + name +  "_psf.fits")
            hdr = hdulist[0].header
            hdr["FWHMMIN"] = str(FHWM_min)
            hdr["FWHMMEA"] = str(FHWM_mean)
            hdr["FWHMMAX"] = str(FHWM_max)
            hdr["PSF_NB"] = str(nb_snaps)
            hdulist.writeto(root + name +  "_psf.fits", overwrite=True)
    
        FWHM_list.append(FHWM_mean)

        mv_p("preppsfex.psf", root + name + ".psf")
        rm_p("preppsfex.psf")
        rm_p("psfex.xml")
    return FWHM_list




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Nov 15 20:08:31 2020.

@author: duverne
"""

from astropy.visualization import ZScaleInterval, imshow_norm, SqrtStretch
import matplotlib.pyplot as plt
import numpy.polynomial.polynomial as poly
import numpy as np
from astropy.table import Table
from astropy.stats import sigma_clip

def plot_image(data, sky_ap=None, ap=None, wcs=None, path2save=None,
               name='/apertures.png'):
    """
    Display the image.

    Parameters
    ----------
    data : image array

    Returns
    -------
    None.

    """
    fig, ax = plt.subplots(1, 1, figsize=(10, 12.5))
    imshow_norm(data, ax,  origin='lower', cmap = 'Greys_r',
                interval=ZScaleInterval(), stretch=SqrtStretch())
    ax.set_xlabel(r'X')
    ax.set_ylabel(r'Y')
    ax.set_title(r'Apertures')
    if ap is not None:
        for aperture in ap:
            aperture.plot(axes=ax, color='red', lw=1.5)
    if sky_ap is not None:
        for aperture in sky_ap:
            aperture.to_pixel(wcs).plot(axes=ax, color='red', lw=1.5)
    if path2save is not None:
        fig.savefig(path2save + name)
        plt.close(fig)

def plot_sementation_image(segmentation, path2save=None):
    """
    Display segmentation image.

    Parameters
    ----------
    segmentation : photutils segmentation objet

    Returns
    -------
    None.

    """
    fig, ax = plt.subplots(1, 1, figsize=(10, 12.5))
    cmap = segmentation.make_cmap(seed=123)
    ax.imshow(segmentation, origin='lower',
              cmap=cmap)
    ax.set_xlabel(r'X')
    ax.set_ylabel(r'Y')
    ax.set_title(r'Segmentation Map')
    if path2save is not None:
        fig.savefig(path2save + '/segmentation.png')
        plt.close(fig)

def plot_background(data, background, dif=False, save=False, path=None):
    """
    Display backgroung image or the result of the background subtraction.

    Parameters
    ----------
    data : image array
    background : background array
    dif : boolean, optional
        wether or not display the result of the background subtraction.
        The default is False.
    save : boolean, optional
        Save the figure.
    path : str, optional
        Path where the figure is saved.

    Returns
    -------
    None.

    """
    if dif:
        fig, ax = plt.subplots(1, 3, figsize=(10, 12.5))
        imshow_norm(data, ax[0],  origin='lower', cmap = 'Greys_r',
                    interval=ZScaleInterval(), stretch=SqrtStretch())
        imshow_norm(background, ax[1],  origin='lower', cmap = 'Greys_r',
                    interval=ZScaleInterval(), stretch=SqrtStretch())
        imshow_norm(data-background, ax[2],  origin='lower', cmap = 'Greys_r',
                    interval=ZScaleInterval(), stretch=SqrtStretch())
    else:
        fig, ax = plt.subplots(1, 1, figsize=(12.5, 12.5))
        im, norm = imshow_norm(background, ax,  origin='lower', cmap = 'Greys_r',
                    interval=ZScaleInterval(), stretch=SqrtStretch())
        ax.set_xlabel(r'$X$')
        ax.set_ylabel(r'$Y$')
        ax.set_title(r'$Background$ $Map$')
        fig.colorbar(im, ax=ax)
        if save:
            try:
                fig.savefig(path)
                plt.close(fig)
            except ValueError:
                raise ValueError("Need a path where to save the plot.")

def plot_calib(band, source, fit_coef, da_2, db_2, path2save=None):
    """
    Plot the calibration curve for an image

    Parameters
    ----------
    band : str
        Filter used to acquire the image.
    source : astropy Table
        Table contatining the calibration informations.
    fit_coef : numpy array, list
        Fit parameters of the calibration.
    path2save : str, optional
        Where to save the figure created if a path is given.
        The default is None.

    Returns
    -------
    None.

    """

    fig, ax = plt.subplots(1, 1, figsize=(12.5, 12.5))

    fit_lin = poly.polyval(source['Magnitude'],
                           np.flip(fit_coef))
    source['mag_err'] = 2.5/(np.log(10)*np.sqrt(source['aperture_sum']))
    da, db = np.sqrt(da_2), np.sqrt(db_2) 
    ax.plot(source['Magnitude'],
            fit_lin,
            label = r'Fit : a={0:.2f} $\pm$ {1:.2f} and zp={2:.2f} $\pm$ {3:.2f}'.format(fit_coef[0], da, 
                                                                                         fit_coef[1], db),
            color='red')
    khi = np.sum(((fit_lin - source[band]) ** 2)/source['error'])
    khi_dof = khi/(len(source['Magnitude']) - 2)
    label = r'$\frac{\chi^2}{dof}$'
    label+= r' = {0:.3f}'.format(khi_dof)
    ax.errorbar(source['Magnitude'], 
                source[band],
                xerr=source['mag_err'],
                yerr=source['error'],
                marker="x",
                ms='9',
                color='black' ,
                linestyle='none',
                label = label)
    ax.set_xlabel(r'Instrumental Magnitude [mag]', fontsize=18)
    ax.set_ylabel(r'Catalog Magnitude [mag]', fontsize=18)
    ax.set_title(r'Calibration Curve', fontsize=18)
    
    ax.xaxis.label.set_size(20)
    ax.yaxis.label.set_size(20)
    ax.grid(visible=True, which='major', color='#666666', linestyle='-')
    ax.minorticks_on()
    
    ax.legend(fontsize=18)
    if path2save is not None:
        fig.savefig(path2save + '/calibration.png')
        plt.close(fig)

def add_error(data_table):

    data_table['err_bkg'] = 2.5/(np.log(10)*np.sqrt(np.abs(data_table['bkg_sum'])))
    data_table['err_bkg_sub'] = 2.5/(np.log(10)*np.sqrt(np.abs(data_table['background_sum'])))
    data_table['err_tot'] = np.sqrt(data_table['err_bkg']**2
                                    + data_table['error']**2
                                    + data_table['err_bkg_sub']**2)
    data_table['star_ref_err_bkg_sub'] = 2.5/(np.log(10)*np.sqrt(np.abs(data_table['star_ref_bkg_sum'])))
    data_table['star_ref_err_tot'] = np.sqrt(data_table['star_ref_error']**2
                                             + data_table['star_ref_err_bkg_sub']**2)
    
    return data_table
    

def is_mag_comptible(data_table):
    
    data_table['star_ref_magnitude_inf'] = (data_table['star_ref_magnitude'] 
                                            - data_table['star_ref_error_tot'])
    data_table['star_ref_magnitude_sup'] = (data_table['star_ref_magnitude'] 
                                            + data_table['star_ref_error_tot'])
    
    data_table['star_ref_magnitude_cata_inf'] = (data_table['star_ref_magnitude_cata'] 
                                                 - data_table['star_ref_error_cata'])
    data_table['star_ref_magnitude_cata_sup'] = (data_table['star_ref_magnitude_cata'] 
                                                 + data_table['star_ref_error_cata'])

    mask1 = (data_table['star_ref_magnitude_sup'] > data_table['star_ref_magnitude_cata_inf'])
    data_table = data_table[mask1]
    mask2 = (data_table['star_ref_magnitude_inf'] < data_table['star_ref_magnitude_cata_sup'])
    data_table = data_table[mask2]

    return data_table

def prepare_light_curve(data_file, clipping_psf=3., err_star=1.0, tele='undef' ):
    
    data_table = Table.read(data_file,
                            format='ascii.commented_header')
    
    bkg_time = np.mean(data_table['time_bkg'])
    photo_time = np.mean(data_table['time_phot_time'])
    xmatch_time = np.mean(data_table['time_xmatch'])
    sub_time = np.mean(data_table['time_sub'])
    im_time = np.mean(data_table['time_image'])
    
    n_im = len(data_table)
    print('{} images processed for {}'.format(len(data_table), tele))
    mask = (data_table['flag_dist'] == 0)
    data_table = data_table[mask]
    print('{} images with detection for {}'.format(len(data_table), tele))
    print('{} images without detection for {}'.format(n_im - len(data_table),
                                                      tele))
    n_im = len(data_table)
    mask = (data_table['star_ref_error_tot'] < err_star)
    data_table = data_table[mask]
    data_table = is_mag_comptible(data_table)
    print('{} images passed star veto for {}'.format(len(data_table), tele))
    print('{} images rejected by star veto'.format(n_im - len(data_table)))
    n_im = len(data_table)
    clip = sigma_clip(data_table['psf'], sigma = clipping_psf, masked=True)
    clip_mask = np.invert(clip.recordmask)
    data_table = data_table[clip_mask]
    print('{} images passed PSF veto for {}'.format(len(data_table), tele))
    print('{} images rejected by PSF veto'.format(n_im - len(data_table)))
    print('{} valid images'.format(len(data_table)))
    
    print('Mean poisson error {} for {}'.format(np.mean(data_table['error']),
                                                tele))
    print('Mean calibration error {} for {}'.format(np.mean(data_table['error_calib']),
                                                    tele))
    print('Mean background error {} for {}'.format(np.mean(data_table['error_bkg']),
                                                   tele))
    
    print('Mean time processing bkg : {} for {}'.format(bkg_time, tele))
    print('Mean time processing photo : {} for {}'.format(photo_time, tele))
    print('Mean time processing xmatch : {} for {}'.format(xmatch_time, tele))
    print('Mean time processing sub image : {} for {}'.format(sub_time, tele))
    print('Mean time processing one image : {} for {}'.format(im_time, tele))
    print('Total time processing {} for : {}'.format(data_table['tot_time'][0],
                                                     tele))

    return data_table

def plot_star_ref(file, telescope, band, clipping_psf=3.0,
                  err_star=1.0, xlabel=r'$Time$', save=False, display=True):
    
    # plot lc star ref #   
    fig, main_axes = plt.subplots(1,1)
    plt.gcf().subplots_adjust(left = 0.1, bottom = 0.1,
                              right = 0.9, top = 0.9,
                              wspace = 2.5, hspace = 0.5)
    
    ##### resultats TCH - bande g #####
    cleaned = prepare_light_curve(file, clipping_psf, err_star)

    ###### for raw plots ####
    raw = Table.read(file, format='ascii.commented_header')
    mask = (raw['flag_dist'] == 0)
    raw = raw[mask]
    
    ### Cleaned plots ###
    main_axes.errorbar(raw['time'],
                        raw['star_ref_magnitude'],
                        yerr=raw['star_ref_error_tot'],
                        label= band + ' ' + r'band' + ' ' + telescope +
                        ' '+'raw',
                        marker = 'x', linestyle = 'none',
                        color = 'red', ms=5)
    main_axes.errorbar(cleaned['time'],
                        cleaned['star_ref_magnitude'],
                        yerr=cleaned['star_ref_error_tot'],
                        label= band + ' ' + r'$band$' + ' ' + telescope,
                        marker = 'd', linestyle = 'none',
                        color = 'blue', ms=7)
    main_axes.errorbar(raw['time'],
                        raw['star_ref_magnitude_cata'],
                        yerr=raw['star_ref_error_cata'],
                        label= r'$Catalog$ $Magnitude = $'+ '%.2f' %raw['star_ref_magnitude_cata'][0],
                        linestyle = '-',
                        color = 'black', ms=5)
    

    main_axes.set_xlabel(xlabel)
    main_axes.set_ylabel(r'$Magnitude$')
    main_axes.set_title(r'$Lightcurve$ $Star$' + ' ' + telescope)
    main_axes.legend()
    main_axes.invert_yaxis()
    if save:
        fig.savefig(band+'star_ref_' + telescope + '.png')
        plt.close(fig)
    if not display:
        plt.close(fig)
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 25 00:35:04 2021

@author: duverne, IJClab, Orsay, duverne@protonmail.com
"""

import argparse, os, hjson, warnings
from astropy.io import fits
from astropy.wcs import WCS, FITSFixedWarning
from astropy.table import Table
from muphoten.utils import (getTel, make_results_dir, get_filename,
                            list_files, load_config, set_results_table)
from muphoten.sanitise import sanitise_fits
from muphoten.catalog import get_filter
from muphoten.mag_lim import compute_mag_lim

warnings.filterwarnings("ignore", category=FITSFixedWarning)
warnings.simplefilter(action="ignore", category=FutureWarning)

"""Muphoten script for limit magnitude estimation."""


def main():

    """Console script for muphoten."""
    # path_muphoten = getpath()
    telescope_list = getTel()
    parser = argparse.ArgumentParser(description="Performing photometry of\
                                     transient objects.")

    parser.add_argument("--images",
                        dest="images",
                        required=True,
                        type=str,
                        help="Path to images.")

    parser.add_argument("--outname",
                        required=True,
                        type=str,
                        help="Name of the output file.")

    parser.add_argument("--telescope",
                        dest="telescope",
                        choices=telescope_list,
                        type=str,
                        help="Telescope that acquired the images.")

    parser.add_argument("--precision",
                        default=0.2,
                        type=float,
                        help="Bin size for the estimation")

    parser.add_argument("--lower-mag",
                        default=15.,
                        type=float,
                        help="Lower bound for the limit magnitude.")

    parser.add_argument("--upper-mag",
                        default=22.,
                        type=float,
                        help="Upper bound for the limit magnitude.")

    parser.add_argument("--threshold",
                        default=0.5,
                        type=float,
                        help="Threshold where the limit magnitude is\
                              considered reached")
    
    args = parser.parse_args()
    
    print('Loading configuration for Muphoten,')
    config = load_config(args.telescope)
    mu = config['muphoten']['conf']
    with open(mu) as json_file:
        mu_conf = hjson.load(json_file)

    images = list_files(args.images, get_subdirs=False)
    results_file = os.path.join(args.images, 'results')

    col_name = ['filename',
                'telescope',
                'filter',
                'time',
                'limit_magnitude',
                'precision',
                'interval_inf',
                'interval_sup']
    res = set_results_table(col_name, len(images))

    for i, image in enumerate(images):
        print(image)
        image = make_results_dir(image,
                                 directory='mag_lim')
        print(image)
        image_rep = os.path.dirname(image)
        calib_rep = os.path.join(image_rep, 'mag_lim')

        image_name = get_filename(image)
        res['filename'][i] = image_name
        path_ratio = os.path.join(calib_rep, 'ratio')
        path_histo = os.path.join(calib_rep, 'histogram')
        print('Processing image n° {} : {}'.format(i, image_name))
        print('Sanitising header')
        # sanitise_fits(image)
        hdu = fits.open(image)[0]
        data, header = hdu.data, hdu.header
        wcs_data = WCS(header)

        if 'TELESCOP' in header.keys():
            res['telescope'][i] = header['TELESCOP']
        else:
            res['telescope'][i] = args.telescope

        # Getting different useful informations : band, MJD,
        # catalog for calibration.
        res['time'][i] = header.get('MJD-OBS')

        band = get_filter(header)
        res['filter'][i] = band

        print('Estimating the limit magnitude in {}'.format(image_name))
        interval = [args.lower_mag, args.upper_mag]
        lim_mag, precision = compute_mag_lim(data, wcs_data, band, mu_conf,
                                             precision=args.precision,
                                             interval=interval,
                                             threshold=args.threshold,
                                             path_ratio=path_ratio,
                                             path_histo=path_histo)
        res['limit_magnitude'][i] = lim_mag
        res['precision'][i] = precision
        res['interval_inf'][i] = interval[0]
        res['interval_sup'][i] = interval[1]
        Table(res[i]).write(calib_rep + '/mag_lim_results.dat',
                     format='ascii.commented_header',
                     overwrite=True)

    res.write(results_file + '/' + args.outname,
              format='ascii.commented_header',
              overwrite=True)
    return 0

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Sun Nov 15 19:48:07 2020.

@author: duverne
"""
import os
from photutils import (MeanBackground, MedianBackground,
                       ModeEstimatorBackground, MMMBackground,
                       SExtractorBackground, BiweightLocationBackground,
                       Background2D)
from astropy.stats import SigmaClip
from muphoten.utils import createFITS
from muphoten.plot_image import plot_background

"""Muphoten main script for background treatment."""

def background_estimation(data, n_sigma=3.0, box_size=(30, 30),
                          bkg_estimator='sex',
                          filter_size=(3, 3), mask=None):
    """
    Estimation of the background of a fits image with photutils\
    background2D method.

    Parameters
    ----------
    data : data from a .fits file
    n_sigma : sigma level from the median
    box_size : size of the in which the BG is estimated
    bkg_estimator : BG level estimator
    filter_size = size of the filter to use

    Returns :
    -------
    Background object of the data.

    """
    sigma_clip = SigmaClip(sigma=n_sigma)
    if bkg_estimator=='sex':
        bkg_estimator = SExtractorBackground()
    elif bkg_estimator=='mean':
        bkg_estimator = MeanBackground()
    elif bkg_estimator=='median':
        bkg_estimator = MedianBackground()
    elif bkg_estimator=='mode':
        bkg_estimator = ModeEstimatorBackground()
    elif bkg_estimator=='mmm':
        bkg_estimator = MMMBackground()
    elif bkg_estimator=='bi':
        bkg_estimator = BiweightLocationBackground()

    bkg = Background2D(data, box_size=box_size,
                       filter_size=filter_size,
                       sigma_clip=sigma_clip,
                       bkg_estimator=bkg_estimator,
                       coverage_mask=mask)
    return bkg

def bkg_to_array(bkg, rms=True):
    """
    Return an array of the background and an array of its root mean square.

    Parameters
    ----------
    bkg : photutils background object
        An estimation of the background in an image.
    rms : Bool
        if True, compute the rms of the background object

    Returns
    -------
    bkg_array : numpy array
        The background image.
    bkg_rms : numpy array
        The image of the root mean square of the background.

    """
    if rms:
        bkg_array = bkg.background
        bkg_rms = bkg.background_rms
        return bkg_array, bkg_rms
    else:
        bkg_array = bkg.background
        return bkg_array

def subtract(data, bkg):
    """
    Subtraction of the background to an image.

    Parameters
    ----------
    data : numpy array
        Data array.
    bkg : numpy array
        background array.

    Returns
    -------
    data_clean : numpy array
        Background subtracted array

    """
    data_clean = data - bkg
    return data_clean

def sub_background(data, header, image_name, n_sigma=3.0, box_size=(30, 30),
                   bkg_estimator='sex', filter_size=(3, 3), mask=None,
                   path2save=None, fits=None, png=False):
    """
    Perform the background subtraction in a fits image.
    Possible to save the background  and the backgrounds subtracted image
    as a fits images.

    Parameters
    ----------
    data : numpy array
        Image as a array.
    header : astropy header object
        Header of the image.
    image_name : str
        name of the image to save the cleaned image and the background.
    n_sigma : float, optional
        sigma level from the median. The default is 3.0.
    box_size : tuple size 2, optional
        Size of the boxe in which the background is estimated.
        The default is (30, 30).
    bkg_estimator : str, optional
        Estimator name. The default is 'sex'.
    filter_size : tuple size 2, optional
        Size of the filter. The default is (3, 3).
    mask : boolean array, optional
        Array used to mask pixels if some regions
        of the image have to be excludes from
        the estimation of the background.
        The default is None.
    path2save : str, optional
        Path where the are saved. The default is None.
    fits : Bool, optional
        Used to save the background as a fits file. The default is False.
    png : Bool, optional
        Used to save the background as a png file. The default is False.

    Returns
    -------
    data_cleaned : numpy array
        Background subtracted image.
    background_array : numpy array
        Background array.
    background_rms : numpy array
        2D Standard devitation of the image.

    """

    # Computing the background with photutils
    bkg_obj = background_estimation(data,
                                    n_sigma = n_sigma,
                                    box_size = box_size,
                                    bkg_estimator=bkg_estimator,
                                    filter_size=filter_size,
                                    mask=mask)

    # Convert the Background object into a numpy array
    background_array, background_rms = bkg_to_array(bkg_obj, rms=True)

    # Subtracting the background to the image
    data_cleaned = data - background_array

    # Save clean image if required
    if path2save is not None:
        background_filename = os.path.join(path2save,
                                           'background/background_' +
                                           image_name)
        if fits:
            clean_filename = os.path.join(path2save,
                                          'clean_' + image_name + '.fits')
            createFITS(data_cleaned, header, filename=clean_filename)
    
        # Save background image

            createFITS(background_array, header,
                       filename=background_filename+'.fits')
    
        # Save RMS image
            RMS_filename = os.path.join(path2save,
                                        'background/rms_' + image_name)
            createFITS(background_rms, header,
                       filename=RMS_filename+'.fits')

        if png:
            plot_background(data, background_array, dif=False,
                            save=True, path=background_filename+'.png')

    return data_cleaned, background_array, background_rms



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug 19 18:41:20 2021

@author: duverne, IJClab, Orsay, duverne@protonmail.com
"""

import os
import subprocess
import hjson
import numpy as np
from astropy.io import fits
from astropy import wcs
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
from astropy import units as u
from muphoten.registration import registration
from muphoten.ps1_survey import ps1_grid, prepare_PS1_sub
from muphoten.utils import ( mkdir_p, rm_p, get_corner_coords, load_config)
# from muphoten.utils import get_phot_cat
from muphoten.psfex import psfex
from muphoten.mosaic import create_mosaic
from muphoten.substraction import hotpants

ima = '/home/duverne/Documents/muphoten/tests/test_im/results/2018cow_20180620_043132_Jun8kevf_kait_B_c/2018cow_20180620_043132_Jun8kevf_kait_B_c.fit'
ref = '/home/duverne/Documents/muphoten/tests/test_im/2018cow_20180807_045955_Aug87fij_kait_B_c_ref.fit'
im_coords = get_corner_coords(ima)
config = load_config('KAIT')
resultDir = '/home/duverne/Documents/muphoten/tests/test_im/results/2018cow_20180620_043132_Jun8kevf_kait_B_c/subtraction'


subfiles = [[ima, ref, None]]


regis_info = registration(subfiles, config, resultDir=resultDir)

sub_files = hotpants(regis_info, config, verbose="QUIET", nb_threads=8)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 17 22:31:08 2021

@author: duverne & Corre, duverne@protonmail.com
"""
import os, os.path
import errno
import glob
import importlib
import shutil
import time
import numpy as np
from astropy.table import Table
from astropy.io import fits
from astropy.wcs import WCS

def is_subdir(path, basepath):
    """ Checks whether the path is inside basepath """
    path = os.path.abspath(path)
    basepath = os.path.abspath(basepath)

    return os.path.commonpath([path, basepath]) == basepath


def is_psf(filename, patterns=['_psf.fits']):
    """
    Check whether a file is a PSF file with a standard name, typically
    '_psf.fits'. This is used for the simulation of sources during the CNN
    training for instance.
    """
    flag = False
    for ptn in patterns:
        if ptn in filename:
            flag = True
            break

    return flag

def list_files(paths, pattern=["*.fit", "*.fits", "*.fts"],
                recursive=True, get_subdirs=True, exclude=None):
    """ (Recursively) list the files matching the pattern from the list of
    filenames or directories, omitting the ones containing file paths
    specified by 'exclude' option (either string or list of strings)"""

    filenames = []
    subdirs = []

    # Make sure paths is array-like
    paths = np.atleast_1d(paths)

    # Check if paths or files provided by user do exist.
    # filenames or folders can be a list
    for path in paths:
        if not os.path.exists(path):
            raise FileNotFoundError(
                errno.ENOENT,
                os.strerror(errno.ENOENT),
                path
            )

    # If there are than one pth provided keep the exact same structure
    # Otherwise skip the basepath
    for path in paths:
        # List all the files in the given path
        if os.path.isdir(path):
            # Recursively get all files matching given pattern(s)
            filenames_path = []

            for ptn in np.atleast_1d(pattern):
                filenames_path += glob.glob(path + "/**/" +
                                            ptn, recursive=recursive)
            # Sort alphanumerically
            filenames_path.sort()
            filenames += filenames_path
            # Get the relative path folder namesrelative to input paths
            reldirs = [os.path.dirname(os.path.relpath(_, path))
                       for _ in filenames_path]
            # When more than one argument, add the base directory
            # being the last directory name provided by the user.
            if len(paths) > 1:
                basedir = os.path.basename(os.path.abspath(path))
                subdirs += [os.path.join(basedir, _) for _ in reldirs]
            else:
                subdirs += reldirs
        else:
            # if path is not a directory, assume it is a file
            filenames += [path]
            subdirs += ['']

    if isinstance(exclude, str):
        folder2skip = [exclude]
    elif exclude:
        folder2skip = exclude
    else:
        folder2skip = []

    idx = []  # Boolean mask whether to keep the filename or not

    for f in filenames:
        flag_2keep = True
        # If file is a standard PSF fits file
        if is_psf(f, patterns=['_psf.fits', 'SUB',
                               'sub', 'conv',
                               'noisemap', 'clean', 'image',
                               'badpix', 'dupe', 'background',
                               'results', '_ref', 'old', '_veto']):
            flag_2keep = False
        else:
            for text in folder2skip:
                if is_subdir(f, text):
                    flag_2keep = False
                    break

        idx.append(flag_2keep)

    idx = np.array(idx)
    filenames = np.array(filenames)
    subdirs = np.array(subdirs)

    if get_subdirs:
        return filenames[idx], subdirs[idx]
    else:
        return filenames[idx]

def prepare_file(path):

    images = list_files(path, get_subdirs=False)
    prepared_list = []
    for image in images:
        image_name = image.split('/')[-1].split('.')[0]
        im = path + image_name
        sub = glob.glob(im +'/subtraction/*.fit*') # adding subtracted image
        if len(sub) > 0:
            sub = sub[0]
        else:
            sub = ''
        psf = glob.glob(im +'/psf/*.fit*') # adding psf file,
        if len(psf) > 0 : # necessary for vetoing
            psf = psf[0]
        else:
            psf = ''
        back = glob.glob(im +'/background/*.fit*')
        if len(back) > 0:
            back = back[0] # adding background file
            clean = back[1] # adding background subtracted image
        else:
            back = ''
            clean = ''
        prepared_list.append([image, sub, psf, back, clean])

    return prepared_list

def load_coord_file(path):

    coordinates = Table.read(path, format='ascii.commented_header')
    transients = coordinates[coordinates['type']=='transient']
    stars = coordinates[coordinates['type']=='star']
    coord_stars, coord_transients = [], []


    for transient in transients:
        coord_transients.append([transient['Ra'], transient['Dec']])
    if coord_transients[0] == 0:
        raise ValueError('No coordinates found for transient.')

    for star in stars:
        coord_stars.append([star['Ra'], star['Dec']])
        coord_stars[0]
    if len(stars) == 0:
        raise ValueError('No coordinates found for reference star.\
                         Needed to veto poor quality images.')
    return coord_transients, coord_stars

def createFITS(data, header, filename = 'image.fits'):
    hdu = fits.PrimaryHDU(data)
    hdul = fits.HDUList([hdu])
    hdul.writeto(filename, overwrite=True)
    fits.writeto(filename, data, header, overwrite=True,
                 output_verify='ignore')

def load_config(telescope, convFilter='default'):
    """Load the path to the configuration files required by the softs.
       They are telescope dependent.
    """
    path = getpath()
    path2tel = os.path.join(path, "config", telescope)
    config = {
        "telescope": telescope,
        "sextractor": {
            "conf": os.path.join(path2tel, "sourcesdet.sex"),
            "param": os.path.join(path2tel, "sourcesdet.param"),
            "convFilter": os.path.join(
                path,
                "config/conv_kernels/%s.conv" % convFilter
            )
        },
        "scamp": {
            "sextractor": os.path.join(path2tel, "prepscamp.sex"),
            "param": os.path.join(path2tel, "prepscamp.param"),
            "conf": os.path.join(path2tel, "scamp.conf"),
        },
        "swarp": {},
        "psfex": {
            "sextractor": os.path.join(path2tel, "preppsfex.sex"),
            "param": os.path.join(path2tel, "preppsfex.param"),
            "conf": os.path.join(path2tel, "psfex.conf"),
        },
        "hotpants": {
            "conf": os.path.join(path2tel, "hotpants.hjson"),
            "conf2": os.path.join(path2tel, "hotpants_2.hjson"),
            "conf3": os.path.join(path2tel, "hotpants_3.hjson"),
        },
        "muphoten": {
            "conf": os.path.join(path2tel, "muphoten.hjson"),
        },
    }

    return config

def getpath():
    """Get the path to muphoten package"""
    try:
        findspec = importlib.util.find_spec("muphoten")
        path = findspec.submodule_search_locations[0]
    except BaseException:
        print("path to muphoten can not be found.")

    return path

def PhotList():
    return ['kron', 'iso', 'fixed']

def getTel():
    """Get the list of all telescopes"""
    path_muphoten = getpath()
    telList = [name for name in os.listdir(os.path.join(path_muphoten, "config"))
               if os.path.isdir(os.path.join(path_muphoten, "config", name)) and
               name != 'conv_kernels']
    return telList

def cp_p(src, dest):
    try:
        shutil.copy(src, dest)
    except BaseException:
        pass


def mv_p(src, dest):
    try:
        shutil.move(src, dest)
    except BaseException:
        pass


def rm_p(src):
    fileList = glob.glob(src, recursive=False)
    for filePath in fileList:
        try:
            os.remove(filePath)
        except BaseException:
            pass

def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

def get_filename(path):
    filename = os.path.splitext(os.path.split(path)[1])[0]
    return filename

def make_results_dir(filename, directory, keep=False, skip=False):
    """ Make the results folder """

    # get name of repertory
    dirname = os.path.dirname(filename)

    results_folder = os.path.join(dirname, 'results')
    if not os.path.exists(results_folder):
        mkdir_p(results_folder)
    dirname = results_folder
    # get filename
    file = os.path.basename(filename)
    # Subdir to store results for this file

    basename = os.path.splitext(file)[0]
    if not basename == os.path.basename(dirname):
        mkdir_p(os.path.join(dirname, basename))
        dirname = os.path.join(dirname, basename)
    # Full path for the file
    newrep = os.path.join(dirname, directory)
    # Make output dir
    if os.path.exists(newrep):
        if skip:
            return None
        elif keep:
            mv_p(newrep, newrep + '_' + time.strftime("%Y%m%d-%H%M%S"))
        else:
            shutil.rmtree(newrep)
    if not os.path.exists(newrep):
        mkdir_p(newrep)
    if not os.path.exists(os.path.join(dirname, file)):
        cp_p(filename, dirname)

    return os.path.join(dirname, file)

def make_sub_result_rep(image_rep, directory='results'):

    newrep = os.path.join(image_rep, directory)
    if not os.path.exists(newrep):
        mkdir_p(newrep)

    return newrep

def get_im_res(rep, image_name, prefix='sub_', hdr=False):

    image = os.path.join(rep, prefix +  image_name + '.fits')
    if hdr:
        hdu = fits.open(image)[0]
        data, header = hdu.data, hdu.header
        return data, header
    else:
        image = fits.open(image)[0].data
        return image

def get_observer(header):

    if 'TELESCOPE' in header.items():
        observer = header['TELESCOPE']
    else:
        observer = 'GRANDMA'

    return observer
    
def set_results_table(col_names, N_rows):
    
    res_table = Table()
    res_table['image_index'] = np.arange(N_rows)
    for name in col_names:
        res_table[name] = None
    
    return res_table

def get_corner_coords(filename):
    """Get the image coordinates of an image"""

    header = fits.getheader(filename)
    # Get physical coordinates
    Naxis1 = header["NAXIS1"]
    Naxis2 = header["NAXIS2"]

    pix_coords = [[0, 0, Naxis1, Naxis1], [0, Naxis2, Naxis2, 0]]

    # Get physical coordinates
    w = WCS(header)
    ra, dec = w.all_pix2world(pix_coords[0], pix_coords[1], 1)

    return [ra, dec]

def clean_outputs(filenames, outLevel):
    """Delete non required output files"""

    imagelist = np.atleast_1d(filenames)
    for ima in imagelist:
        print("\nCleaning up output files for %s" % ima)
        path = os.path.dirname(ima)
        print(ima)

        rm_p(os.path.join(path, "*.head"))
        if outLevel == 0:
            rm_p('coadd.weight.fits')
            files = glob.glob(path+'/subtraction/*.fits')
            for f in files:
                if '_sub' not in f:
                    rm_p(f)
            for _ in ["*.fits",
                      "*.fit",
                      "*.sh",
                      "subtraction/*mask*",
                      "subtraction/*background",
                      "subtraction/*segmentation"]:
                rm_p(os.path.join(path, _))

        elif outLevel == 1:
            rm_p('coadd.weight.fits')

            for _ in ["subtraction/*mask*",
                      "subtraction/*background",
                      "subtraction/*segmentation"]:
                rm_p(os.path.join(path, _))

        elif outLevel == 2:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Sun Nov 15 00:09:43 2020.

@author: duverne
"""

import warnings
import numpy as np
from astropy.stats import (gaussian_fwhm_to_sigma)
from astropy.convolution import Gaussian2DKernel
from astropy import units as u
# from astropy.table import vstack, hstack, join
# from astropy import units as u, coordinates as coord
from astropy.wcs import WCS, FITSFixedWarning
from photutils import (detect_sources, deblend_sources, source_properties,
                       SkyEllipticalAperture, aperture_photometry,
                       SkyCircularAperture, CircularAperture,
                       EllipticalAperture)
from photutils.utils import calc_total_error

warnings.filterwarnings("ignore", category=FITSFixedWarning)


def masking(image, coordinate, header, mask_size):
    """
    Create a square mask around a position.

    Parameters
    ----------
    image : numpy array
        Image data.
    coordinate : list of length 2
        position in radec for the center of the mask.
    header : header object
        header of the fits file.
    mask_size : int
        size of the mask in pixel.

    Returns
    -------
    mask : boolean numpy array
        boolean mask.

    """
    w = WCS(header)
    coord_image = w.all_world2pix(coordinate, 1)
    shape = np.shape(image)
    xs = coord_image[0][0]
    ys = coord_image[0][1]
    mask = np.ones_like(image, dtype=bool)
    yo = max(int(ys) - int(mask_size), 0)
    y1 = min(int(ys) + int(mask_size), shape[0])
    xo = max(int(xs) - int(mask_size), 0)
    x1 = min(int(xs) + int(mask_size), shape[1])
    print(xo, x1)
    print(yo, y1)
    mask[yo:y1, xo:x1] = False

    return mask


def source_detection(data, background_array, background_rms, n_sigma=3.0,
                     deblend=True, mask=None, threshold_level=2.,
                     npixels=5, contrast=0.001):
    """
    Detect the sources in an image.

    Parameters
    ----------
    data : numpy array
        Image data NOT background subtracted.
    background_array : numpy array
        Background in the image data.
    background_rms : numpy array
        Background RMS in the image data.
    n_sigma : int, optional
        FWHM of the gaussian in pixel used to filter the data.
        The default is 3.0.
    deblend : Boolean, optional
        Deblend or not the sources. The default is True.
    mask : numpy array, optional
        A mask to define where to search for sources. The default is None.
    threshold_level : Float, optional
        Level to above the background to define a source.
        The default is 2. sigma.
    npixels : int, optional
        Number of connected pixels to define a source. The default is 5.
    contrast : Float, optional
        Fraction of the total source flux that a local peak must have to be
        considered as a separate object. The default is 0.001.

    Returns
    -------
    Photutils Segmentation object
        A segmentation image containing the detected sources in the image.

    """
    # WARNING data MUST NOT BE background subtracted
    threshold = background_array + (threshold_level*background_rms)
    sigma = n_sigma * gaussian_fwhm_to_sigma
    kernel = Gaussian2DKernel(sigma, x_size=3, y_size=3)
    kernel.normalize()
    if mask is not None:
        segm = detect_sources(data, threshold, filter_kernel=kernel,
                              npixels=npixels, mask=mask)
    else:
        segm = detect_sources(data, threshold, npixels=npixels,
                              filter_kernel=kernel)
    if deblend:
        if segm is not None:
            segm = deblend_sources(data, segm, npixels=npixels,
                                   filter_kernel=kernel,
                                   contrast=contrast)
    return segm


def Fixed_Photometry(raw_data, sub_data, segmentation, header, radius,
                     background_array, background_rms, wcs=None):
    """

    """
    try:
        time = float(header.get("EXPOSURE"))
    except Exception:
        try:
            time = float(header.get("EXPTIME"))
        except Exception:
            time = 1.0

    error = calc_total_error(raw_data,
                             background_rms,
                             effective_gain=time)
    cat = source_properties(sub_data, segmentation,
                            background=background_array,
                            error=error, wcs=wcs)
    columns = ['id', 'xcentroid', 'ycentroid', 'sky_centroid',
               'background_sum']#, 'centroid']
    table = cat.to_table(columns=columns)
    table['xcentroid'].info.format = '.4f'  # optional format
    table['ycentroid'].info.format = '.4f'
    apertures = SkyCircularAperture(table['sky_centroid'], r=radius*u.pix)
    photometry = aperture_photometry(sub_data, apertures.to_pixel(wcs))
    table['aperture'] = apertures
    table['aperture_sum'] = photometry['aperture_sum']
    table['Magnitude'] = -2.5 * np.log10(table['aperture_sum'])
    table['Magnitude'].info.format = '.4f'
    table['idx'] = np.arange(len(table))

    return table


def Kron_Photometry(raw_data, sub_data, segmentation, header,
                    background_array, background_rms, wcs=None):
    """
    Perform photometry in an image.

    Parameters
    ----------
    raw_data : numpy array
        Image data, background NOT subtracted.
    sub_data : numpy array
        Image data, background subtracted.
    segmentation : photutils segmentation object
        Segmented image.
    header : FITS header
        Header of the FITS file.
    background_array : numpy array
        Background of the image.
    background_rms : numpy array
        Background RMS of the image.
    wcs : None or WCS object, optional
        A world coordinate system (WCS) transformation.
        If None, then all sky-based properties will be set to None.

    Returns
    -------
    table : astropy table
        Astropy table with all the informations about the
        sources detected in data.

    """
    # WARNING data MUST NOT BE BG substracted here
    try:
        time = float(header.get("EXPOSURE"))
    except Exception:
        try:
            time = float(header.get("EXPTIME"))
        except Exception:
            time = 1.0

    error = calc_total_error(raw_data,
                             background_rms,
                             effective_gain=time)
    # WARNING data MUST BE BG substracted here
    cat = source_properties(sub_data, segmentation,
                            background=background_array,
                            error=error, wcs=wcs,
                            kron_params=('correct', 2.5, 0.001, 'exact', 5))
    columns = ['id', 'xcentroid', 'ycentroid', 'source_sum', 'area',
               'background_at_centroid', 'background_mean', 'background_sum',
               'sky_centroid', 'kron_aperture', 'kron_flux', 'kron_radius']
    table = cat.to_table(columns=columns)
    table['xcentroid'].info.format = '.4f'  # optional format
    table['ycentroid'].info.format = '.4f'
    table['source_sum'].info.format = '.4f'
    table['background_at_centroid'].info.format = '{:.4f}'  # optional format
    table['background_mean'].info.format = '{:.4f}'
    table['background_sum'].info.format = '{:.4f}'
    table['kron_flux'].info.format = '{:.4f}'
    table['aperture_sum'] = table['kron_flux']
    table['kron_radius'].info.format = '{:.4f}'
    table['aperture'] = table['kron_aperture']
    table['Magnitude'] = -2.5 * np.log10(table['kron_flux'])
    table['Magnitude'].info.format = '.4f'
    table['idx'] = np.arange(len(table))

    return table


def Isophotal_Photometry(raw_data, sub_data, segmentation, header,
                         background_array, background_rms,
                         radius=4, wcs=None):
    """  Compute the instrumental magnitude and several other properties of
    a fits file. This is done using aperture photometry

    Parameters
    ----------
    data : data from a .fits file
    segm_deblend : segmentation image with the sources detected
    header : header of the fits file
    background : estimaton of the background of data
    n_pixels_in : semi-major axis of the inner ellipsis for
    aperture photometry
    n_pixels_out : semi-major axis of the extern ellipsis
    for aperture photometry

    Returns : (background) of the data
    -------
    """

    # WARNING data MUST NOT BE BG substracted here
    try:
        time = float(header.get("EXPOSURE"))
    except Exception:
        try:
            time = float(header.get("EXPTIME"))
        except Exception:
            time = 1.0
    error = calc_total_error(raw_data,
                             background_rms,
                             effective_gain=time)
    cat = source_properties(sub_data, segmentation,
                            background=background_array,
                            error=error, wcs=wcs)
    columns = ['id', 'xcentroid', 'ycentroid',
               'sky_centroid', 'background_sum',
               'semimajor_axis_sigma', 'semiminor_axis_sigma', 'orientation']
    table = cat.to_table(columns=columns)
    table['xcentroid'].info.format = '.4f'  # optional format
    table['ycentroid'].info.format = '.4f'
    apertures = []
    photometry = []
    for i in range(len(table)):
        elli = SkyEllipticalAperture(positions=table['sky_centroid'][i],
                                     a=table['semiminor_axis_sigma'][i]*radius,
                                     b=table['semimajor_axis_sigma'][i]*radius,
                                     theta=table['orientation'][i])
        apertures.append(elli)
        photometry.append(aperture_photometry(sub_data,
                                              elli.to_pixel(wcs))['aperture_sum'][0])
    table['aperture'] = apertures
    table['aperture_sum'] = photometry
    table['Magnitude'] = -2.5 * np.log10(table['aperture_sum'])
    table['Magnitude'].info.format = '.4f'
    table['idx'] = np.arange(len(table))

    return table


def get_source(table, coord_source, wcs, col_name='dist'):

    coord_source = wcs.all_world2pix(coord_source, 1)
    table[col_name] = np.sqrt((table['xcentroid']/u.pix -
                               coord_source[0][0])**2
                              + (table['ycentroid']/u.pix -
                                 coord_source[0][1])**2)
    source = table[table[col_name] == min(table[col_name])]
    return source


def Dophot(data, position, wcs, radius=5.0, aperture=None):

    if aperture is None:
        position = wcs.all_world2pix(position, 1)
        aperture = CircularAperture(position, r=radius)
        aper_photo = aperture_photometry(data, aperture)
    else:
        if isinstance(aperture,EllipticalAperture):
            aperture.to_sky(wcs).to_pixel(wcs)
        else:
            aperture = aperture.to_pixel(wcs)
        aper_photo = aperture_photometry(data, aperture)

    return aper_photo['aperture_sum'][0]


def compute_error(table, a, da_2, db_2):

    table['poisson_source_error'] = 2.5/(np.log(10) *
                                         np.sqrt(table['aperture_sum']))
    table['poisson_bkg_error'] = 2.5/(np.log(10) *
                                      np.sqrt(table['background_sum']))
    table['calib_error'] = np.sqrt(table['Magnitude']**2 * da_2 +
                                   db_2 +
                                   a * table['poisson_source_error']**2)

    table['total_error'] = np.sqrt(table['poisson_source_error']**2 +
                                   table['poisson_bkg_error']**2 +
                                   table['calib_error']**2)
    print(table['total_error'])
    return table


def compute_snr(table):

    table['SNR'] = (table['aperture_sum'])/np.sqrt(table['aperture_sum'] +
                                                   table['background_sum'])

    return table



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Sun Apr 18 18:11:45 2021

@author: duverne, IJClab, Orsay, duverne@protonmail.com
"""

import argparse
import sys
import os
import warnings
from astropy.io import fits
import hjson

from muphoten.background import sub_background
from muphoten.utils import (list_files, get_filename, make_results_dir,
                            getTel, load_config)

warnings.simplefilter(action="ignore", category=FutureWarning)

"""Muphoten script for background substraction."""

def main():
    """Muphoten background subtraction."""

    telescope_list = getTel()

    parser = argparse.ArgumentParser(description="Subtracts background in fits\
                                     images using photutils. Can ave the\
                                     background subtracted images or the\
                                     background image.")

    parser.add_argument("--images",
                        required=True,
                        type=str,
                        help="Path where the images are stored. ")

    parser.add_argument("--telescope",
                        choices=telescope_list,
                        required=True,
                        type=str,
                        help="Alias for the available telescopes.")

    parser.add_argument("--keep-old",
                        dest="keep",
                        required=False,
                        action="store_true",
                        help="Keep previous results")

    parser.add_argument("--skip-processed",
                        "--skip",
                        dest="skip",
                        required=False,
                        action="store_true",
                        help="Skip already processed files")

    parser.add_argument("--save-png",
                        action="store_true",
                        help="Save the background images as fits and png.\
                              Default : False")

    parser.add_argument("--save-fits",
                        action="store_true",
                        help="Save the background images as fits and png.\
                              Default : False")

    args = parser.parse_args()

    print('Loading configuration for Muphoten,')
    config = load_config(args.telescope)
    mu =config['muphoten']['conf']
    with open(mu) as json_file:
        mu_config = hjson.load(json_file)

    # Loading the image to analyse
    images = list_files(args.images)[0]
    print('Processing {} images'.format(len(images)))
    print(images)
    for i, image in enumerate(images):

        image = make_results_dir(image,
                                directory='background',
                                keep=args.keep,
                                skip=args.skip)
        image_name = get_filename(image)
        print('Subtracting background in image n° {} : {}'.format(i,
                                                                  image_name))
        hdu = fits.open(image)[0]
        data, header = hdu.data, hdu.header

        path2save=os.path.dirname(image)

        sub_background(data, header, image_name,
                       n_sigma=mu_config['bkg_n_sigma'],
                       box_size=mu_config['bkg_box'],
                       bkg_estimator=mu_config['bkg_estimator'],
                       filter_size=mu_config['bkg_filter_size'],
                       path2save=path2save, fits=args.save_fits, png=args.save_png)

    return 0

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Dec  7 13:13:10 2020

@author: duverne
"""

from astropy.io import fits
from astropy.time import Time  
import glob

def add_filter(fits_file, band):
    
    with fits.open(fits_file, mode = 'update') as hdul:
        hdul[0].header.append(('FILTER', band))

def add_MJD(fits_file, mjd):

    with fits.open(fits_file, mode = 'update') as hdul:
        hdul[0].header.append(('MJD-OBS', mjd))


file = '/home/duverne/gmadet/final_kepd/gmadet_results/ATLAS18qqn_0_g_20180620_043424.017130_o/ATLAS18qqn_0_g_20180620_043424.017130_o.fits'

# list_file = glob.glob('/home/duverne/Documents/AT2018cow_data/kped/g_band'+'/*')
list_file = glob.glob('/media/duverne/DISQUE ESSB/AT2018cow/kepd/r_band'+'/*')
list_file=[file for file in list_file if '_psf' not in file and 'sub_' not in file]

for file in list_file:

    # add_filter(file, 'g')
    name = file.split('/')[-1]
    obs = name.split('_')
    
    y = obs[3][0:4]
    m = obs[3][4:6]
    d = obs[3][6:8]
    time = y + '-' + m + '-' + d +'T'
    
    h = obs[4][0:2]
    mi = obs[4][2:4]
    sec = obs[4][4:]
    time+=h+':'+mi+':'+sec
    time_utc = Time(time, format='isot')
    mjd = time_utc.mjd
    print(mjd)
    add_MJD(file, mjd)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Fri May  7 01:39:48 2021

@author: corre, duverne, IJClab, Orsay, duverne@protonmail.com
"""

import numpy as np
from astropy.io import fits
from astropy.time import Time, TimeDelta

def sanitise_headers(filename, fix=False):
    """
    Keep only important keywords.
    Helps not to crash when headers are not well formated.
    """

    #  Need to keep minimum information about astrometric
    #  calibration for initialising scamp
    #  List of official keywords:
    #  https://heasarc.gsfc.nasa.gov/docs/fcg/standard_dict.html
    # Other keywords commonly used:
    #  https://heasarc.gsfc.nasa.gov/docs/fcg/common_dict.html
    #  Keywords not present in this list are discarded.
    keywords_to_keep = [
        "SIMPLE",
        "BITPIX",
        "NAXIS",
        "NAXIS1",
        "NAXIS2",
        "EXTEND",
        "DATE-OBS",
        "DATE-END",
        "TELESCOP",
        "INSTRUME",
        "OBJECT",
        "EXPTIME",
        "FILTER",
        "GAIN",
        "SATURATE",
        "EQUINOX",
        "EPOCH",
        # "RADESYS",
        "CTYPE1",
        "CTYPE2",
        "CUNIT1",
        "CUNIT2",
        "CRVAL1",
        "CRVAL2",
        "CRPIX1",
        "CRPIX2",
        "CD1_1",
        "CD1_2",
        "CD2_1",
        "CD2_2",
        "CDELT1",
        "CDELT2",
        "CROTA1",
        "CROTA2",
        "BSCALE",
        "BZERO",
        "BUNIT",
        "AIRMASS",
        "END",
        "MJD-OBS",
        'JD',
        'DATE',
        'BJD-OBS',
        'HJD-OBS',
        'JD-OBS',
        'OBSDATE',
        'TELESCOPE',
        'INSTFILT']
    hdulist = fits.open(filename)
    # Verify and try to fix issue with fits standard
    if fix: 
        hdulist.verify("fix")
    hdr = hdulist[0].header
    keywords2delete = []
    for key, value in hdr.items():
        #  if EXPOSURE keyword present, rename it EXPTIME
        if key == "EXPOSURE":
            #  check if EXPTIME exists, otherwise create it
            try:
                if hdr["EXPTIME"]:
                    pass
            except BaseException:
                hdr["EXPTIME"] = value
                #  if EXPOSURE keyword present, rename it EXPTIME
        if key == "FILTERS":
            #  check if FILTER exists, otherwise create it
            try:
                if hdr["FILTER"]:
                    pass
            except BaseException:
                hdr["FILTER"] = value

        if key not in keywords_to_keep:
            keywords2delete.append(key)
    keywords2delete = np.unique(keywords2delete)
    for key in keywords2delete:
        del hdr[key]
    if 'EXPTIME' not in hdr:
        start, end = Time(hdr['DATE-OBS']), Time(hdr['DATE-END'])
        hdr["EXPTIME"] = int(TimeDelta([end.jd-start.jd],
                                       format='jd').to_value(format='sec')[0])
        # Try to set the exposure time if not in headern with start and
        # end time in the header.

    if 'FILTER' not in hdr:
        if "INSTFILT" in hdr:
            hdr["FILTER"] = hdr["INSTFILT"]
        else:
            raise KeyError('No filter found in {}.'
                           'Use the set-band command to modify the'
                           'header.'.format(filename))
    hdulist.writeto(filename, overwrite=True)

def sanitise_data(filename):
    """
    Make sure that the data are in the primary hdu.
    Otherwise some astromatic softs are not working properly.
    Other solution would be to check astromatic soft config.
    """
    # Make sure to use only the Primary hdu.
    # sextractor, scamp seems to crash otherwise.
    hdul = fits.open(filename)
    # hdul.verify('fix')
    if len(hdul) > 1:
        print("More than one HDU in fits file.")
        print("Keeping only the PrimaryHDU.")
        newhdu = fits.PrimaryHDU()
        newhdu.data = hdul[0].data
        newhdu.header = hdul[0].header
        newhdulist = fits.HDUList([newhdu])
        newhdulist.writeto(filename, overwrite=True)
        hdul.close()

def sanitise_time(filename):
    """
    Set the MJD-OBS keyword in the header for the analysis

    Parameters
    ----------
    filename : path.
        The path to an image to check its header and add
        the keyword if necessary.

    Returns
    -------
    None.

    """
    hdulist = fits.open(filename)
    header = hdulist[0].header
    times = ['JD',#'DATE',
             'DATE-OBS','OBSDATE',
             'BJD-OBS', 'HJD-OBS', 'JD-OBS']
    if 'MJD-OBS' not in header:
        for time in times:
            if time in header:
                try:
                    MJD = Time(header[time]).mjd
                    header["MJD-OBS"] = "{}".format(MJD)
                    print(MJD)
                    break
                except ValueError:
                    continue
        hdulist.writeto(filename, overwrite=True)

def add_filter(filename, band):
    """
    Add the FILTER keyword in header.

    Parameters
    ----------
    fits_file : Path.
        Image in which the keyword must be added.
    band : str
        Filter that has been used to acquire the image.

    Returns
    -------
    None.

    """    
    with fits.open(filename, mode = 'update') as hdul:
        hdul[0].header.append(('FILTER', band))

def add_observer(filename, tel):
    """
    Add the observer in the fits header

    Parameters
    ----------
    filename : fits file
        File to be modified..
    tel : str
        Name opf the observer
    Returns
    -------
    None.

    """
    with fits.open(filename, mode = 'update') as hdul:
        hdul[0].header.append(('TELESCOP', tel))

def sanitise_fits(filename):
    """Call function to sanitise fits headers and data"""
    try:
        sanitise_headers(filename)
    except Exception:
        sanitise_headers(filename, fix=True)
    sanitise_headers(filename)
    sanitise_time(filename)
    sanitise_data(filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 22 23:11:05 2022

@author: duverne, IJClab, Orsay, duverne@protonmail.com
"""

from .background_subtraction import *
from .mag_lim import *
from .photometry import *
from .psf import *
from .sanitise import *
from .subtraction import *
